<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/lib/toolslib.php - tools
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wascore
 * @version $Id: toolslib.php,v 1.14 2016/06/28 14:08:33 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }

/* This is the list of recognised tasks and chores in Tools */

define('TASK_TOOLS_INTRO','intro');
define('TASK_TRANSLATETOOL','translatetool');
define('TASK_BACKUPTOOL','backuptool');
define('TASK_SESSIONTOOL','sessiontool');
define('CHORE_SESSION_VIEW','view');
define('CHORE_SESSION_DELETE','delete');
define('TASK_LOGVIEW','logview');

/** main entry point for tools (called from /program/main_admin.php)
 *
 * this routine dispatches the tasks, If the specified task
 * is not recognised, the default task TASK_TOOLS_INTRO
 * is executed.
 *
 * @param object &$output collects the html output
 * @return void results are returned as output in $output
 * @todo fix permissions for backup tool! perhaps another bit?
 */
function job_tools(&$output) {
    global $CFG,$WAS_SCRIPT_NAME,$USER;
    $output->set_helptopic('tools');
    $task = get_parameter_string('task',TASK_TOOLS_INTRO);
    switch($task) {
    case TASK_TOOLS_INTRO:
        show_tools_intro($output);
        show_tools_menu($output);
        break;

    case TASK_TRANSLATETOOL:
        if ($USER->has_job_permissions(JOB_PERMISSION_TRANSLATETOOL)) {
            include($CFG->progdir.'/lib/translatetool.class.php');
            $mgr = new TranslateTool($output);
            if ($mgr->show_parent_menu()) {
                show_tools_menu($output,$task);
            }
        } else {
            $output->add_content("<h2>".t('access_denied','admin')."</h2>");
            $output->add_content(t('job_access_denied','admin'));
            $output->add_message(t('job_access_denied','admin'));
            show_tools_menu($output,$task);
        }
        break;

    case TASK_BACKUPTOOL:
        if ($USER->has_job_permissions(JOB_PERMISSION_BACKUPTOOL)) {
            task_backuptool($output);
        } else {
            $output->add_content("<h2>".t('access_denied','admin')."</h2>");
            $output->add_content(t('job_access_denied','admin'));
            $output->add_message(t('job_access_denied','admin'));
            show_tools_menu($output,$task);
        }
        break;

    case TASK_SESSIONTOOL:
        if ($USER->has_job_permissions(JOB_PERMISSION_SESSIONTOOL)) {
            task_sessiontool($output);
            show_tools_menu($output,$task);
        } else {
            $output->add_content("<h2>".t('access_denied','admin')."</h2>");
            $output->add_content(t('job_access_denied','admin'));
            $output->add_message(t('job_access_denied','admin'));
            show_tools_menu($output,$task);
        }
        break;

    case TASK_LOGVIEW:
        if ($USER->has_job_permissions(JOB_PERMISSION_LOGVIEW)) {
            task_logview($output);
        } else {
            $output->add_content("<h2>".t('access_denied','admin')."</h2>");
            $output->add_content(t('job_access_denied','admin'));
            $output->add_message(t('job_access_denied','admin'));
            show_tools_menu($output,$task);
        }
        break;

    default:
        $s = (utf8_strlen($task) <= 50) ? $task : utf8_substr($task,0,44).' (...)';
        $message = t('task_unknown','admin',array('{TASK}' => htmlspecialchars($s)));
        $output->add_message($message);
        logger('tools: unknown task: '.htmlspecialchars($s));
        show_tools_intro($output);
        show_tools_menu($output);
        break;
    }
} // job_tools()


/** display an introductory text for tools + menu
 *
 * @param object &$output collects the html output
 * @return void results are returned as output in $output
 */
function show_tools_intro(&$output) {
        $output->add_content('<h2>'.t('tools_header','admin').'</h2>');
        $output->add_content(t('tools_intro','admin'));
} // task_tools_intro()


/** display the tools menu
 *
 * @param object &$output collects the html output
 * @param string $current_task indicate the current menu selection (if any)
 * @return void results are returned as output in $output
 */
function show_tools_menu(&$output,$current_task=NULL) {
    global $WAS_SCRIPT_NAME,$USER;
    $menu_items = array(
        array(
            'task' => TASK_TRANSLATETOOL,
            'anchor' => t('menu_translatetool','admin'),
            'title' => t('menu_translatetool_title','admin'),
            'permission' => JOB_PERMISSION_TRANSLATETOOL
        ),
        array(
            'task' => TASK_BACKUPTOOL,
            'anchor' => t('menu_backuptool','admin'),
            'title' => t('menu_backuptool_title','admin'),
            'permission' => JOB_PERMISSION_BACKUPTOOL
        ),
        array(
            'task' => TASK_SESSIONTOOL,
            'anchor' => t('menu_sessiontool','admin'),
            'title' => t('menu_sessiontool_title','admin'),
            'permission' => JOB_PERMISSION_SESSIONTOOL
        ),
        array(
            'task' => TASK_LOGVIEW,
            'anchor' => t('menu_logview','admin'),
            'title' => t('menu_logview_title','admin'),
            'permission' => JOB_PERMISSION_LOGVIEW
        )
    );
    $output->add_menu('<h2>'.t('menu','admin').'</h2>');
    $output->add_menu('<ul>');
    foreach($menu_items as $item) {
        $parameters = array('job' => JOB_TOOLS, 'task' => $item['task']);
        $attributes = array('title' => $item['title']);
        if (!($USER->has_job_permissions($item['permission']))) {
            $attributes['class'] = ($current_task == $item['task']) ? 'dimmed current' : 'dimmed';
        } elseif ($current_task == $item['task']) {
            $attributes['class'] = 'current';
        }
        $output->add_menu('  <li>'.html_a($WAS_SCRIPT_NAME,$parameters,$attributes,$item['anchor']));
    }

    // Updater is located in a separate file and called automagically
    // whenever there is a version mismatch. However, we also provide
    // a 'manual' entrance here. 2010-12-20/PF
    $parameters = array('job' => JOB_UPDATE);
    $attributes = array('title' => t('menu_update_title','admin'));
    if (!($USER->has_job_permissions(JOB_PERMISSION_UPDATE))) {
        $attributes['class'] = 'dimmed';
    }
    $output->add_menu('  <li>'.html_a($WAS_SCRIPT_NAME,$parameters,$attributes,t('menu_update','admin')));
    $output->add_menu('</ul>');
} // show_tools_menu()


/** show an introductory text for backup tool OR stream a ZIP-file to the browser
 *
 * If we arrive here via the tools menu, the parameter download is not set.
 * In that case we show an introductory text with a link that yields a ZIP-file
 * with the backup.
 *
 * If the user follows the download link, we arrive here too but with the download
 * parameter set. We then dump the database in a variable and subsequently 
 * compress it in a ZIP-file which we stream to the browser. We do code some
 * things in the basename of the backup:
 *  - the hostname
 *  - the database name
 *  - the database prefix
 *  - the date and the time
 * which should be enough to distinguish nearly all backups if you happen to have a
 * lot of different ones. Note that the URL is also encoded as a comment in the .ZIP.
 *
 * The parameter download is currently set to 'zip'. However, we do attempt to send the
 * plain uncompressed data if that parameter is set to 'sql' (quick and dirty). Oh well.
 * hopefully there is enough memory to accomodate backups of moderate sized sites.
 *
 * Note that we need space to compress the data; a informal test yielded that we need
 * about 160% of the uncompressed size of the backup (tested with a small testset).
 * Rule of the thumb for memory: the more the merrier but at least twice the size of the
 * uncompressed backup.
 *
 * @param object &$output collects output to show to user
 * @return output displayed via $output OR ZIP-file streamed to the browser
 */
function task_backuptool(&$output) {
    global $DB,$CFG,$WAS_SCRIPT_NAME;
    $download = get_parameter_string('download',NULL);
    if (is_null($download)){
        $output->set_helptopic('backuptool');
        $output->add_content("<h2>".t('backuptool_header','admin')."</h2>");
        $params = array('{DATADIRECTORY}' => htmlspecialchars($CFG->datadir));
        $output->add_content(t('backuptool_intro','admin',$params));
        $output->add_content('<p>');
        $parameters = array('job' => JOB_TOOLS, 'task' => TASK_BACKUPTOOL, 'download' => 'zip');
        $attributes = array('title' => t('backuptool_download_title','admin'));
        $anchor = t('backuptool_download','admin');
        $output->add_content(html_a($WAS_SCRIPT_NAME,$parameters,$attributes,$anchor));
        show_tools_menu($output,TASK_BACKUPTOOL);
        return;
    }
    $host = tools_get_hostname();
    // basename = host "-" database "-" prefix "-" timestamp (only acceptable chars for filename; strip fancy chars)
    $basename = $host.
                '-'.trim(preg_replace('/[^a-zA-Z0-9._-]/','',$CFG->db_name),'-_.').
                '-'.trim(preg_replace('/[^a-zA-Z0-9._-]/','',$CFG->prefix ),'-_.').
                '-'.strftime('%Y%m%d-%H%M%S');
    $archive  = $basename.'.zip';
    $backup   = $basename.'.sql';
    $data     = '';
    logger(sprintf('%s(): start creating/streaming backup %s',__FUNCTION__,$basename),WLOG_DEBUG);
    if ($DB->dump($data)) {
        $data_length = strlen($data);
        logger(sprintf('%s(): backup file %s is %d bytes',__FUNCTION__,$backup,$data_length),WLOG_DEBUG);
        if ($download == 'sql') {
            header('Content-Type: text/x-sql');
            header(sprintf('Content-Disposition: attachment; filename="%s"',$backup));
            logger(sprintf('%s(): about to stream %d bytes (backup = %s)',__FUNCTION__,$data_length,$backup),WLOG_DEBUG);
            echo $data;
            logger(sprintf('%s(): success streaming data %s (%d bytes uncompressed)',__FUNCTION__,$backup,$data_length));
            exit;
        } else {
            include_once($CFG->progdir.'/lib/zip.class.php');
            $zip = new Zip;
            $comment = $CFG->www;
            if ($zip->OpenZipstream($archive,$comment)) {
                if ($zip->AddData($data,$backup)) {
                    if ($zip->CloseZip()) {
                        logger(sprintf('%s(): success streaming backup %s (%d bytes uncompressed)',
                                        __FUNCTION__,$archive,$data_length));
                     } else {
                         logger(sprintf('%s(): cannot close zipstream %s: %s',__FUNCTION__,$archive,$zip->Error));
                     }
                } else {
                    logger(sprintf('%s(): cannot add backup data to zipstream %s: %s',__FUNCTION__,$archive,$zip->Error));
                    $zip->CloseZip();
                }
                // ZipStream is done, no point in adding garbage at the end of that file, quit completely
                exit;
            } else {
                logger(sprintf('%s(): cannot open zipstream %s for backup: %s',__FUNCTION__,$archive,$zip->Error));
            }
        }
    } else {
        logger(sprintf('%s(): cannot open dump data for backup: %s',__FUNCTION__,db_errormessage()));
    }
    $output->add_message(t('backuptool_error','admin'));
    show_tools_menu($output);
    show_tools_intro($output);
} // task_backuptool()


/** main entry point for session management
 *
 * @param object &$output collects the html output
 * @return void results are returned as output in $output
 */
function task_sessiontool(&$output) {
    $output->set_helptopic('sessiontool');
    $chore = get_parameter_string('chore',CHORE_SESSION_VIEW);
    switch($chore) {
    case CHORE_SESSION_VIEW:
        sessions_show_overview($output,NULL,TRUE);
        break;

    case CHORE_SESSION_DELETE:
        $session_id = get_parameter_int('session',0);
        sessions_delete($output,$session_id);
        break;

    default:
        $s = (utf8_strlen($task) <= 50) ? $task : utf8_substr($task,0,44).' (...)';
        $message = t('chore_unknown','admin',array('{CHORE}' => htmlspecialchars($s)));
        $output->add_message($message);
        logger('sessions: unknown chore: '.htmlspecialchars($s));
        sessions_show_overview($output);
        break;
    }
    return;
} // task_sessiontool()


/** show an overview of currently existing sessions
 *
 * @param object &$output collects the html output
 * @return void results are returned as output in $output
 */
function sessions_show_overview(&$output) {
    $output->add_content('<h2>'.t('sessions_overview','admin').'</h2>');
    sessions_show($output);
    $output->add_content('<br>');
} // session_show_overview()


/** show a table with sessions
 *
 * display a table with sessions and locked nodes per session
 *
 * If $id is NULL, all current sessions+locks are displayed,
 * if $id is a specific session_id only that session is displayed.
 * (used in the session delete chore, see {@link sessions_delete()}).
 *
 * If $show_delete is TRUE a link is displayed that allows the user
 * to delete a session, if FALSE no link is displayed.
 *
 * @param object &$output collects the html output
 * @param mixed $id select which session(s) to display (Null means all)
 * @param bool $show_delete if TRUE show clickable links to delete option
 * @return mixed FALSE on error, number of selected sessions otherwise
 */
function sessions_show(&$output,$id=NULL,$show_delete=TRUE) {
    global $DB;
    $where = (is_null($id)) ? '' : sprintf('WHERE s.session_id = %d ',$id);
    $sql = sprintf("SELECT s.session_id, s.user_id,s.user_information,s.ctime,s.atime,".
                   "u.username,u.full_name,n.node_id, n.locked_since,n.is_page,n.title,n.link_text ".
                   "FROM %ssessions s ".
                   "LEFT JOIN %susers u USING(user_id) ".
                   "LEFT JOIN %snodes n ON s.session_id = n.locked_by_session_id ".
                   "%s".
                   "ORDER BY s.session_id,n.locked_since",$DB->prefix,$DB->prefix,$DB->prefix,$where);
    if (($DBResult = $DB->query($sql)) === FALSE) {
        logger(sprintf('%s(): error retrieving sessions data: %s',__FUNCTION__,db_errormessage()));
        $output->add_message(t('error_retrieving_data','admin'));
        $output->add_content(t('sessions_not_found','admin'));
        return FALSE;
    }
    if ($DBResult->num_rows < 1) {
        $output->add_content(t('sessions_not_found','admin'));
        return 0;
    }
    $last_id = 0;
    $oddeven = 'header';
    sessions_show_header($output,$oddeven,$show_delete);
    while (($record = $DBResult->fetch_row_assoc()) !== FALSE) {
        if (($session_id = intval($record['session_id'])) != $last_id) {
            $last_id = $session_id;
            sessions_show_session($output,$record,$oddeven,$show_delete);
        }
        if (!is_null($record['node_id'])) {
            sessions_show_node_locked($output,$record,$oddeven,$show_delete);
        }
    }
    sessions_show_footer($output);
    return $DBResult->num_rows;
} // sessions_show()

/** show the header of the HTML-table
 *
 * <pre>
 * | D | sss | ctime | atime | username (full_name) | ip_addr |
 * |     nnn | ltime | link_text (title)                      |
 * </pre>
 *
 * the HTML-table has 5 or 6 columns, depending on $show_delete.
 * the header takes two rows: one for the session information,
 * and another for the node lock information.
 *
 * Parameter $class is used for styling of the table header.
 *
 * @param object &$output collects the html output
 * @param string &$class attribute of table rows and cells
 * @param bool $show_delete if TRUE show clickable links to delete option
 * @return void html written to $output
 */
function sessions_show_header(&$output,&$class,$show_delete=TRUE) {
    $attributes = array('class' => $class);
    $left = array('class' => $class,'align' => 'left');
    $right = array('class' => $class,'align' => 'right');
    $output->add_content(html_table(array('cellpadding' => '3')));
    $output->add_content('  '.html_table_row($attributes));
    if ($show_delete) {
        $output->add_content('    '.html_table_head($attributes,'&nbsp;'));
    }
    $output->add_content('    '.html_table_head($right,t('sessions_session_id','admin')));
    $output->add_content('    '.html_table_head($left,t('sessions_cdatim','admin')));
    $output->add_content('    '.html_table_head($left,t('sessions_adatim','admin')));
    $output->add_content('    '.html_table_head($left,t('sessions_username','admin')));
    $output->add_content('    '.html_table_head($left,t('sessions_ip_addr','admin')));
    $output->add_content('  '.html_table_row_close());
    $output->add_content('  '.html_table_row($attributes));
    if ($show_delete) {
        $right['colspan'] = '2';
    }
    $output->add_content('    '.html_table_head($right,t('sessions_node_id','admin')));
    $output->add_content('    '.html_table_head($left,t('sessions_ldatim','admin')));
    $left['colspan'] = '3';
    $output->add_content('    '.html_table_head($left,t('sessions_link','admin')));
    $output->add_content('  '.html_table_row_close());
} // sessions_show_header()


/** show session information from a single (combined) record
 *
 * the HTML-table has 5 or 6 columns, depending on $show_delete.
 * Parameter $class is used for styling of the table row.
 * The value is toggled between 'odd' and 'even'.
 *
 * @param object &$output collects the html output
 * @param array &$record contains session and lock information
 * @param string &$class attribute of table rows and cells
 * @param bool $show_delete if TRUE show clickable links to delete option
 * @return void html written to $output
 */
function sessions_show_session(&$output,&$record,&$class,$show_delete=TRUE) {
    global $WAS_SCRIPT_NAME;

    // 1 -- prepare
    $class = ($class == 'odd') ? 'even' : 'odd';
    $session_id = intval($record['session_id']);
    $session_self = ($_SESSION['session_id'] == $session_id) ? TRUE : FALSE;
    $attributes = array('class' => $class);
    $left = array('class' => $class,'align' => 'left');
    $right = array('class' => $class,'align' => 'right');
    $username = htmlspecialchars(sprintf('%s (%s)',$record['username'],$record['full_name']));
    $ip_addr = htmlspecialchars($record['user_information']);

    // 2 -- output a new table row with session information
    $output->add_content('  '.html_table_row($attributes));
    if ($show_delete) {
        if ($session_self) {
            $delete = '&nbsp;';
        } else {
            $title = t('icon_session_delete','admin');
            $alt = t('icon_session_delete_alt','admin');
            $text =  t('icon_session_delete_text','admin');
            $anchor = $output->skin->get_icon('delete', $title, $alt, $text);
            $a_params = array('job' => JOB_TOOLS,
                              'task' => TASK_SESSIONTOOL,
                              'chore' => CHORE_SESSION_DELETE,
                              'session' => strval($session_id));
            $attr = array('title' => $title);
            $delete = '<b>'.html_a($WAS_SCRIPT_NAME,$a_params,$attr,$anchor).'</b>';
        }
        $output->add_content('    '.html_table_cell($attributes,$delete));
    }
    $output->add_content('    '.html_table_cell($right,'<b>'.$session_id.'</b>'));
    $output->add_content('    '.html_table_cell($left, '<b>'.short_datim($record['ctime']).'</b>'));
    $output->add_content('    '.html_table_cell($left, '<b>'.short_datim($record['atime']).'</b>'));
    $output->add_content('    '.html_table_cell($left, '<b>'.$username.'</b>'));
    $output->add_content('    '.html_table_cell($left, '<b>'.$ip_addr.'</b>'));
    $output->add_content('  '.html_table_row_close());
} // sessions_show_session()


/** show locked node information from a single (combined) record
 *
 * the HTML-table has 5 or 6 columns, depending on $show_delete.
 * Parameter $class is used for styling of the table row.
 * The value is toggled between 'odd' and 'even'.
 *
 * @param object &$output collects the html output
 * @param array &$record contains session and lock information
 * @param string &$class attribute of table rows and cells
 * @param bool $show_delete if TRUE show clickable links to delete option
 * @return void html written to $output
 */
function sessions_show_node_locked(&$output,&$record,&$class,$show_delete=TRUE) {
    // 1 -- prepare
    $class = ($class == 'odd') ? 'even' : 'odd';
    $attributes = array('class' => $class);
    $left = array('class' => $class,'align' => 'left');
    $right = array('class' => $class,'align' => 'right');
    $link = htmlspecialchars(sprintf('%s (%s)',$record['link_text'],$record['title']));
    $ip_addr = htmlspecialchars($record['user_information']);
    if ($show_delete) {
        $right['colspan'] = '2';
    }
    // 2 -- output table row
    $output->add_content('  '.html_table_row($attributes));
    $output->add_content('    '.html_table_cell($right,$record['node_id']));
    $output->add_content('    '.html_table_cell($left,short_datim($record['locked_since'])));
    $left['colspan'] = '3';
    $output->add_content('    '.html_table_cell($left,$link));
    $output->add_content('  '.html_table_row_close());
} // sessions_show_node_locked()

/** close the HTML-table
 *
 * @param object &$output collects the html output
 * @return void html written to $output
 */
function sessions_show_footer(&$output) {
    $output->add_content(html_table_close());
} // sessions_show_footer()


/** handle confirmation and actual delete of a session
 *
 * this single routine handles
 *
 *  - the display of a confirmation message,
 *  - handling of the actual delete, and
 *  - handling of the bail out/cancelbutton
 *
 * @param object &$output collects the html output
 * @param int $session_id
 * @return void output written via $output
 */
function sessions_delete(&$output,$session_id) {
    global $WAS_SCRIPT_NAME,$CFG,$DB;

    // 0A -- sanity check
    if ($session_id == $_SESSION['session_id']) {
        $output->add_message(t('sessions_cannot_delete_self','admin'));
        sessions_show_overview($output);
        return;
    }

    // 0B -- initialise
    $dialogdef = array(
        'csrftoken' => dialog_csrftoken(),
        'button_delete' => dialog_buttondef(BUTTON_DELETE),
        'buton_cancel' => dialog_buttondef(BUTTON_CANCEL)
        );

    // 1 -- bail out?
    if (isset($_POST['button_cancel'])) {
        $output->add_message(t('cancelled','admin'));
        sessions_show_overview($output);
        return;
    }

    // 2 -- already in confirmation phase?
    if (isset($_POST['button_delete'])) {
        if (!dialog_validate($dialogdef)) {
            foreach($dialogdef as $k => $item) {
                if ((isset($item['errors'])) && ($item['errors'] > 0)) {
                    $output->add_message($item['error_messages']);
                }
            }
        } else {
            // 2A -- start with log message with suitable information
            $errors = 0;
            $msg = sprintf("%s(): session %d forcefully terminated",__FUNCTION__,$session_id);
            $sql = sprintf("SELECT s.*, u.username ".
                           "FROM %ssessions s ".
                           "LEFT JOIN %susers u USING(user_id) ".
                           "WHERE session_id = %d",$DB->prefix,$DB->prefix,$session_id);
            if ((($DBResult = $DB->query($sql)) !== FALSE) && ($DBResult->num_rows == 1)) {
                if (($record = $DBResult->fetch_row_assoc()) !== FALSE) {
                    $msg .= sprintf(" (%d seconds) [login %s(%d) from %s on %s, last access %s]",
                                    $CFG->session_expiry,
                                    empty($record['username']) ? '?' : $record['username'],
                                    $record['user_id'],
                                    $record['user_information'],
                                    $record['ctime'],
                                    $record['atime']);
                } else {
                    ++$errors;
                    $msg .= sprintf(" (no data: %s)",db_errormessage());
                }
            } else {
                ++$errors;
                $msg .= sprintf(" (no data: %s)",db_errormessage());
            }

            // 2B -- first reset the locked pages (if any)
            $table = 'nodes';
            $fields = array('locked_since' => NULL, 'locked_by_session_id' => NULL);
            $where = array('locked_by_session_id' => $session_id);
            if (($locks = db_update($table,$fields,$where)) === FALSE) {
                ++$errors;
                $msg .= sprintf('; node unlock failed: %s',db_errormessage());
            } else {
                $msg .= sprintf('; %d node(s) unlocked',$locks);
            }
            // 2C -- now delete the session record
            $table = 'sessions';
            $where = array('session_id' => $session_id);
            if (($retval = db_delete($table,$where)) === FALSE) {
                ++$errors;
                $msg .= sprintf("; session delete failed: %s",db_errormessage());
            } else {
                $msg .= sprintf("; session %d removed",$session_id);
            }
            logger($msg);
            // 2D -- tell user what happened (more details are in the log)
            if ($errors == 0) {
                $output->add_message(t('success_saving_data','admin'));
            } else { 
                $output->add_message(t('errors_saving_data','admin',array('{ERRORS}' => $errors)));
            }
            sessions_show_overview($output);
            return;
        }
    }
    
    // 3 -- show existing data in confirmation dialogue
    $output->add_content('<h2>'.t('sessions_session_delete','admin').'</h2>');
    $output->add_content('<p>'.t('sessions_session_delete_confirm','admin').'</p>');
    $num = sessions_show($output,$session_id,FALSE); // FALSE=don't show delete link in this table
    $output->add_content('<p>'.t('delete_are_you_sure','admin').'</p>');
    if (($num !== FALSE) && ($num > 0)) {
        $a_params = array('job' => JOB_TOOLS,
                          'task' => TASK_SESSIONTOOL,
                          'chore' => CHORE_SESSION_DELETE,
                          'session' => strval($session_id));
        $href = href($WAS_SCRIPT_NAME,$a_params);
        $output->add_content(dialog_quickform($href,$dialogdef));
    } // else 
        // something went horribly wrong but there is no button to wreak havoc
} // sessions_delete()


/** quick and dirty logfile view / download / prune tool
 *
 * this is the dispatcher that decides whether to show
 * entries, download entries or prune the logs
 *
 * @param object &$output collects output to show to user
 * @return void output displayed via $output
 * @todo should we allow for fancy selection mechanisms on the logfile or is that over the top?
 */
function task_logview(&$output) {
    $download = get_parameter_int('download',NULL);
    if (!is_null($download)) {
        return logview_download($output,$download);
    }
    $prune = get_parameter_int('prune',NULL);
    if (!is_null($prune)) {
        return logview_prune($output,$prune);
    }
    return logview_show($output);
} // task_logview()


/** show entries from log table in a neat HTML-table (possibly paginated)
 *
 * this constructs a table of the HTML-variety with the contents of the logtable.
 * fields displayed are: datim, IP-address, username, logpriority and message
 * we use a LEFT JOIN in order to get to a meaningful username rather than a numeric user_id
 * an attempt is made to start with the last page of the logs because that would probably
 * be the most interesting part. We paginate the log in order to keep it manageable.
 *
 * @param object &$output collects output to show to user
 * @param null|int $offset indicates where to start the listing, NULL=consult GET parameters
 * @return void output displayed via $output
 */
function logview_show(&$output,$offset=NULL) {
    global $CFG,$WAS_SCRIPT_NAME,$DB;

    // 0 -- at least we allow the user to navigate away if something goes wrong
    $output->set_helptopic('logview');
    show_tools_menu($output,TASK_LOGVIEW);

    // 1A -- how many messages are there anyway?
    $table = 'log_messages';
    $where = ''; // could be used to select per user, per priority, etc. For now: always select everything
    if (($record = db_select_single_record($table,'COUNT(log_message_id) AS messages',$where)) === FALSE) {
        $output->add_content('<h2>'.t('menu_logview','admin').'</h2>');
        $output->add_content(t('logview_error','admin'));
        $output->add_message(t('logview_error','admin'));
        logger(sprintf('%s(): cannot retrieve log message count: %s',__FUNCTION__,db_errormessage()));
        return;
    }
    // 1B -- if there are no message we leave
    if (($num_messages = intval($record['messages'])) < 1) {
        $output->add_content('<h2>'.t('menu_logview','admin').'</h2>');
        $output->add_content(t('logview_no_messages','admin'));
        $output->add_message(t('logview_no_messages','admin'));
        logger(sprintf('%s(): no messages to show',__FUNCTION__),WLOG_DEBUG);
        return;
    }

    // 2 -- calculate/retrieve offset and limit)
    $limit = max(1,get_parameter_int('limit',$CFG->pagination_height)); // limit > 0
    if (is_null($offset)) {
        $offset = intval(floor($num_messages  / $limit)) * $limit; // attempt to start at LAST page
        $offset = get_parameter_int('offset',$offset); // ...unless user requested otherwise
    } // else use the parameter supplied
    $offset = max(0,min($offset,$num_messages-1)); // make sure 0 <= $offset < $num_rows

    // 3 -- maybe add pagination and add '[nn-mm of xx]' to header
    $header = t('menu_logview','admin');
    if (($num_messages > $limit) || ($offset != 0)) {
        $parameters = array('job' => JOB_TOOLS, 'task' => TASK_LOGVIEW);
        $output->add_pagination($WAS_SCRIPT_NAME,$parameters,$num_messages,$limit,$offset,
                                $CFG->pagination_width);
        $header .=' '.t('pagination_count_of_total','admin',array('{FIRST}' => $offset+1,
                              '{LAST}' => min($num_messages,$offset+$limit),'{TOTAL}' => $num_messages));
    }

    // 4 -- retrieve the selected messages (including optional username via LEFT JOIN)
    $sql = sprintf('SELECT l.*, u.username FROM %slog_messages l LEFT JOIN %susers u USING (user_id) '.
                   'ORDER BY l.log_message_id', $DB->prefix,$DB->prefix);
    if (($DBResult = $DB->query($sql,$limit,$offset)) === FALSE) {
        $output->add_message(t('logview_error','admin'));
        logger(sprintf('%s(): cannot retrieve log messages: %s',__FUNCTION__,db_errormessage()));
        return;
    }

    // 5A -- header + open HTML table
    $output->add_content('<h2>'.$header.'</h2>');
    $output->add_content('<p>');
    $class = 'header';
    $attributes = array('class' => $class,'align' => 'right');
    $output->add_content(html_table(array('cellpadding' => '3')));
    $output->add_content('  '.html_table_row($attributes));
    $output->add_content('    '.html_table_head($attributes,t('logview_nr','admin')));
    $attributes['align'] = 'left';
    $output->add_content('    '.html_table_head($attributes,t('logview_datim','admin')));
    $output->add_content('    '.html_table_head($attributes,t('logview_remote_addr','admin')));
    $output->add_content('    '.html_table_head($attributes,t('logview_user_id','admin')));
    $output->add_content('    '.html_table_head($attributes,t('logview_priority','admin')));
    $output->add_content('    '.html_table_head($attributes,t('logview_message','admin')));
    $output->add_content('  '.html_table_row_close());

    // 5B -- step through the recordset and dump into the table
    $index = $offset+1;
    $pivot = 0;
    $pivot_date = '';
    while (($record = $DBResult->fetch_row_assoc()) !== FALSE) {
        if ($index == $offset+1) {
            $pivot = $record['log_message_id']; // remember pkey of 1st record for download/prune lateron
            $pivot_date = $record['datim'];
        }
        $class = ($class == 'odd') ? 'even' : 'odd';
        $attributes = array('class' => $class);
        $output->add_content('  '.html_table_row($attributes));
        $attributes['align'] = 'right';
        $attributes['title'] = $record['log_message_id']; // convey real record id in title in 1st column
        $output->add_content('    '.html_table_cell($attributes,strval($index++)));
        $attributes = array('class' => $class);
        $output->add_content('    '.html_table_cell($attributes,htmlspecialchars($record['datim'])));
        $output->add_content('    '.html_table_cell($attributes,htmlspecialchars($record['remote_addr'])));
        $output->add_content('    '.html_table_cell($attributes,htmlspecialchars($record['username'])));
        $output->add_content('    '.html_table_cell($attributes,logview_priority($record['priority'])));
        $output->add_content('    '.html_table_cell($attributes,htmlspecialchars($record['message'])));
        $output->add_content('  '.html_table_row_close());
    }
    $output->add_content(html_table_close());
    $output->add_content('<p>');
    $DBResult->close();

    // 5C -- link to download/prune (only if there are 'older' records, on a previous page)
    if (0 < $offset) {
        $aparams = array('{DATIM}' => $pivot_date, '{COUNT}' => $offset);
        // Download
        $parameters = array('job' => JOB_TOOLS, 'task' => TASK_LOGVIEW, 'download' => $pivot);
        $attributes = array('title' => t('logview_download_title','admin',$aparams));
        $anchor = t('logview_download','admin',$aparams);
        $output->add_content(html_a($WAS_SCRIPT_NAME,$parameters,$attributes,$anchor));
        // (nnn messages before yyyy-mm-dd)
        $output->add_content(t('logview_count_messages','admin',$aparams));
        // Prune
        $parameters = array('job'=>JOB_TOOLS, 'task'=>TASK_LOGVIEW, 'offset'=>$offset, 'prune'=>$pivot);
        $attributes = array('title' => t('logview_prune_title','admin',$aparams));
        $anchor = t('logview_prune','admin',$aparams);
        $output->add_content(html_a($WAS_SCRIPT_NAME,$parameters,$attributes,$anchor));
    }
} // logview_show()


/** download records from the log table in CSV format
 *
 * this sends the contents of the logtable upto (but excluding)
 * the record with pkey = $pivot to the user in CSV format
 *
 * the suggested filename contains the name of the host and
 * the first and the last pkey of the records exported. This
 * allows for downloading logs from different hosts and
 * multiple times from the same host without name clashes.
 *
 * If there are errors or no records nothing is sent to the user
 * (but we do try to log the event).
 *
 * Note
 * The number Nr displayed in the overview (see {@link logview_show()})
 * is just a number starting at 1 every time. The download file
 * works with the real record pkey.
 *
 * @param object &$output collects output to show to user
 * @param int $pivot download limit: pkey must be < $pivot
 * @return void CSV output directly, $output suppressed
 */
function logview_download(&$output,$pivot) {
    global $DB;
    $output->set_suppress_output(TRUE);  // download takes care of output itself

    // 1 -- select the interesting records (if any)
    $sql = sprintf('SELECT l.*, u.username FROM %slog_messages l LEFT JOIN %susers u USING (user_id) '.
                   'WHERE l.log_message_id < %d ORDER BY l.log_message_id',
                   $DB->prefix,$DB->prefix,$pivot);
    if (($DBResult = $DB->query($sql)) === FALSE) {
        logger(sprintf('%s(): cannot retrieve download: %s',__FUNCTION__,db_errormessage()));
        return;
    } elseif ($DBResult->num_rows <= 0) {
        $DBResult->close();
        logger(sprintf('%s(): no records for download: %s',__FUNCTION__,db_errormessage()));
        return;
    }

    // 2 -- export the selected records
    $first = TRUE;
    while (($record = $DBResult->fetch_row_assoc()) !== FALSE) {
        // 2A -- late start with http headers because of using pkey of first record in $filename
        if ($first) {
            $first = FALSE;
            header('Content-Type: text/plain; charset=UTF-8');
            $filename = sprintf('%s-messages-%d-%d.txt',
                                tools_get_hostname(),$record['log_message_id'],$pivot-1);
            header(sprintf('Content-Disposition: attachment; filename="%s"',$filename));
            $eol = "\n";
            $crlf = array("\n\r","\r\n","\r"); 
            echo 'id,"datim","addr","username","prio","message"'.$eol;
            logger(sprintf('%s(): start export of %d records; filename="%s"',
                           __FUNCTION__,$DBResult->num_rows,$filename));
        }
        // 2B -- output CSV-line
        echo sprintf('%d,"%s","%s","%s","%s","%s"%s',
                     $record['log_message_id'],
                     str_replace('"','""',str_replace($crlf,"\n",trim($record['datim']))),
                     str_replace('"','""',str_replace($crlf,"\n",trim($record['remote_addr']))),
                     str_replace('"','""',str_replace($crlf,"\n",trim($record['username']))),
                     logview_priority($record['priority']),
                     str_replace('"','""',str_replace($crlf,"\n",trim($record['message']))),
                     $eol);
    }
    $DBResult->close();
    return;
} // logview_download()


/** ask for confirmation and/or maybe execute pruning of log messages table
 *
 * shows confirmation screen including download link and also actually process
 * the deletions if the user confirmed. We give the user another opportunity
 * to download the selected records before pruning.
 *
 * @param object &$output collects output to show to user
 * @param int $pivot download limit: pkey must be < $pivot
 * @return void output written to $output
 */
function logview_prune(&$output,$pivot) {
    global $WAS_SCRIPT_NAME;
    $table = 'log_messages';

    // 0 -- sanity checks
    $offset = get_parameter_int('offset',NULL);
    if (is_null($offset)) {
        $output->add_message(t('error_invalid_parameters','admin'));
        logview_show($output);
        return;
    }
    if (($record = db_select_single_record($table,'datim',array('log_message_id' => $pivot))) === FALSE) {
        logger(sprintf('%s(): cannot retrieve record at pivot = %d: %s',
                       __FUNCTION__,$pivot,db_errormessage()));
        $output->add_message(t('error_retrieving_data','admin'));
        logview_show($output);
        return;
    }
    $pivot_date = $record['datim'];
    $aparams = array('{DATIM}' => $pivot_date, '{COUNT}' => $offset);

    // 1 -- bail out?
    if (isset($_POST['button_cancel'])) {
        $output->add_message(t('cancelled','admin'));
        logview_show($output);
        return;
    }

    // 2 -- handle the confirmation dialog
    $dialogdef = array(
        'pivot' => array('type' => F_INTEGER,'name' => 'pivot','value' => $pivot,'hidden' => TRUE),
        'offset' => array('type' => F_INTEGER,'name' => 'offset','value' => $offset,'hidden' => TRUE),
        'csrftoken' => dialog_csrftoken(),
        'button_delete' => dialog_buttondef(BUTTON_DELETE),
        'buton_cancel' => dialog_buttondef(BUTTON_CANCEL)
        );

    if (isset($_POST['button_delete'])) {
        if ((dialog_validate($dialogdef)) && 
            ($dialogdef['pivot']['value'] == $pivot) &&
            ($dialogdef['offset']['value'] == $offset)) {
            // 2A -- we are already in the confirmation phase; user confirmed it
            $where = sprintf('log_message_id < %d',$pivot);
            if (($retval = db_delete($table,$where)) === FALSE) {
                $output->add_message(t('logview_prune_errors','admin',$aparams));
                logger(sprintf('%s(): error pruning %d records from %s: %s',
                               __FUNCTION__,$offset,$table,db_errormessage()));
                logview_show($output); // show log at the current position because nothing was deleted(?)
            } else {
                $aparams['{COUNT}'] = $retval;
                $output->add_message(t('logview_prune_success','admin',$aparams));
                logger(sprintf('%s(): success pruning %d of %d records before %s (pivot = %d)',
                               __FUNCTION__,$retval,$offset,$pivot_date,$pivot));
                logview_show($output,0); // show log from the (new) top at offset = 0
            }
            return;
        }
        $output->add_message(t('error_invalid_parameters','admin'));
        foreach($dialogdef as $k => $item) {
            if ((isset($item['errors'])) && ($item['errors'] > 0)) {
                $output->add_message($item['error_messages']);
            }
        }
    }
    // 2B -- show clean confirm dialogue (1st time) OR confirm dialogue with errors for a retry
    $output->add_content('<h2>'.t('logview_prune_confirm','admin',$aparams).'</h2>');
    $output->add_content(html_tag('p','',t('logview_prune_explanation','admin',$aparams)));
    $parameters = array('job' => JOB_TOOLS, 'task' => TASK_LOGVIEW, 'download' => $pivot);
    $attributes = array('title' => t('logview_download_title','admin',$aparams));
    $anchor = t('logview_download','admin',$aparams);
    $output->add_content(html_tag('p','',html_a($WAS_SCRIPT_NAME,$parameters,$attributes,$anchor).' '.
                                         t('logview_count_messages','admin',$aparams)));
    $output->add_content(html_tag('p','',t('delete_are_you_sure','admin')));
    $parameters = array('job'=>JOB_TOOLS, 'task'=>TASK_LOGVIEW, 'offset'=>$offset, 'prune'=>$pivot);
    $href = href($WAS_SCRIPT_NAME,$parameters);
    $output->add_content(dialog_quickform($href,$dialogdef));
} // logview_prune()


/** helper routine to construct something that looks like this hosts name
 *
 * @return string
 */
function tools_get_hostname() {
    global $CFG;
    // Try to construct a suitable hostname as part of the filename.
    $host = '';
    if (($url = parse_url($CFG->www)) !== FALSE) {
        $host = (isset($url['host'])) ? trim(preg_replace('/[^a-zA-Z0-9.-]/','',$url['host']),'.') : '';
    }
    if ((empty($host)) && (isset($_SERVER['SERVER_NAME']))) {
        $host = trim(preg_replace('/[^a-zA-Z0-9.-]/','',$_SERVER['SERVER_NAME']),'.');
    }
    if (empty($host)) {
        $host = 'localhost'; // last resort
    }
    return $host;
} // tools_get_hostname()


/** helper routine to translate a numeric priority to human readable text
 *
 * look up the symbolic name of priority $p
 *
 * [Rant]<br>
 * I used to use the built-in constants like LOG_INFO and LOG_DEBUG to allow for different levels
 * of logging (see {@link logger()}). To my complete surprise logging didn't work at all on
 * Windows (it did on Linux). The reason was that LOG_DEBUG and LOG_INFO and LOG_NOTICE are all
 * defined to be the same value. WTF? Any test based on LOG_DEBUG and LOG_INFO being different
 * would fail, hence no logging at all. The mind boggles! So, instead of using built-in constants
 * I had to define my own and do a global search&replace. Aaarghhhhhh!!!!<br>
 * [/Rant]<br>
 *
 * @param int $p numeric value priority
 * @return string readable priority
 */
function logview_priority($p) {
    static $priorities = array(
        WLOG_EMERG   => 'LOG_EMERG',
        WLOG_ALERT   => 'LOG_ALERT',
        WLOG_CRIT    => 'LOG_CRIT',
        WLOG_ERR     => 'LOG_ERR',
        WLOG_WARNING => 'LOG_WARNING',
        WLOG_NOTICE  => 'LOG_NOTICE',
        WLOG_INFO    => 'LOG_INFO',
        WLOG_DEBUG   => 'LOG_DEBUG');
    return (isset($priorities[$p])) ? $priorities[$p] : strval(intval($p));
} // logview_priority()


?>