<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/aggregator/aggregator_install.php - installer of the aggregator module
 *
 * This file contains the aggregator module installer.
 * The interface consists of these functions:
 *
 * <code>
 * aggregator_install(&$messages,$module_id)
 * aggregator_upgrade(&$messages,$module_id)
 * aggregator_uninstall(&$messages,$module_id)
 * aggregator_demodata(&$messages,$module_id,&$config,$manifest)
 * </code>
 *
 * These functions can be called from the main installer and/or admin.php.
 *
 * Note
 * You cannot be sure about the environment from which these routines are called.
 * If the caller is the Install Wizard, you do not have all subroutines available.
 * However, it IS possible to manipulate the database via the db_*() routines and/or
 * the global $DB object. Therefore you have to keep the install routine extremly
 * simple. You also have no option to interact with the user; the install has to
 * be a silent install; you can only indicate success (TRUE) or failure (FALSE)
 * and maybe an error message in $messages[] but that's it. Good luck.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_aggregator
 * @version $Id: aggregator_install.php,v 1.8 2016/06/16 13:48:31 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }


/** install the module
 *
 * this routine installs the module. For this module
 * there is nothing to install, so we simply return success.
 * The appropriate table is already created based on the tabledefs);
 * see install/aggregator_tabledefs.php.
 *
 * Note that the record for this module is already created in the
 * modules table; the pkey is $module_id.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function aggregator_install(&$messages,$module_id) {
    return TRUE;
} // aggregator_install()


/** upgrade the module
 *
 * this routine performs an upgrade to the installed module.
 * Note that this 'aggregator' module currently does not need any
 * upgrade at all because there never was an earlier version.
 *
 * However, if there was to be a newer version of this module, this
 * routine is THE place to bring the database up to date compared with
 * the existing version. For example, if an additional field 'aggregator_extension'
 * was to be added to the aggregator-table, it could be added using a
 * suitable (default) value, e.g. an empty string or whatever
 *
 * Any existing aggregator could then be updated here to fill the new
 * field with data, e.g.
 *
 * UPDATE aggregator SET aggregator_extension = '';
 *
 * etcetera. For now this routine is a nop.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function aggregator_upgrade(&$messages,$module_id) {
    return TRUE;
} // aggregator_upgrade()


/** uninstall the module
 *
 * this is a hook for future extensions of Website@School.
 * For now we simply return success. Any real code could look like this:
 * DELETE FROM aggregator;
 * to delete all existing data (but who would want that). For
 * now this is simply a nop.
 *
 * Note that bluntly deleting from the aggregator table might lead to
 * nodes without a valid module. The better way to do it would be
 * something like this:
 * <code>
 * SELECT count(node_id) AS number_of_nodes FROM aggregator;
 * 
 * if ($number_of_nodes > 0) then
 *     $messages[] = 'There are still $number_of_nodes nodes with a aggregator';
 *     return FALSE;
 * </code>
 * which in fact means that the table should already be empty before
 * we can empty it. Oh well...
 * 
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function aggregator_uninstall(&$messages,$module_id) {
    return TRUE;
} // aggregator_uninstall()


/** add demonstration data to the system
 *
 * this routine adds to the existing set of demonstration data as specified
 * in $config.
 *
 * for the sake of demonstration we add the following in the showcase section:
 * aggregator
 * demo-news (hidden section)
 *   news 4
 *   news 3
 *   news 2
 *   news 1
 * we subsequently aggregate the demo-news section (which is hidden)
 * we also try to add the snapshots to the list of nodes to aggregate,
 * but that may prove to fail if aggregator is installed before snapshots.
 * Mmmm, must fix that lateron.
 *
 * The array $config contains the following information.
 *
 * <code>
 * $config['language_key']   => install language code (eg. 'en')
 * $config['dir']            => path to CMS Root Directory (eg. /home/httpd/htdocs)
 * $config['www']            => URL of CMS Root Directory (eg. http://exemplum.eu)
 * $config['progdir']        => path to program directory (eg. /home/httpd/htdocs/program)
 * $config['progwww']        => URL of program directory (eg. http://exemplum.eu/program)
 * $config['datadir']        => path to data directory (eg. /home/httpd/wasdata/a1b2c3d4e5f6)
 * $config['user_username']  => userid of webmaster (eg. wblader)
 * $config['user_full_name'] => full name of webmaster (eg. Wilhelmina Bladergroen)
 * $config['user_email']     => email of webmaster (eg. w.bladergroen@exemplum.eu)
 * $config['user_id']        => numerical user_id (usually 1)
 * $config['demo_salt']      => password salt for all demodata accounts
 * $config['demo_password']  => password for all demodata accounts
 * $config['demo_areas']     => array with demo area data
 * $config['demo_groups']    => array with demo group data
 * $config['demo_users']     => array with demo user data
 * $config['demo_nodes']     => array with demo node data
 * $config['demo_string']    => array with demo strings from /program/install/languages/LL/demodata.php
 * $config['demo_replace']   => array with search/replace pairs to 'jazz up' the demo strings
 * </code>
 *
 * With this information, we can add a demonstration configuration for the public area,
 * which shows off the possibilities. Note that we add our own additions to the array
 * $config so other modules and themes can determine the correct status quo w.r.t. the
 * demodata nodes etc.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @param array &$config pertinent data for the new website + demodata foundation
 * @param array $manifest a copy of the manifest for this module
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function aggregator_demodata(&$messages,$module_id,&$config,$manifest) {
    global $DB;
    $retval = TRUE; // assume success

    // 0 -- get hold of the module-specific translations in $string[]
    $string = array();
    $filename = dirname(__FILE__).'/languages/en/aggregator.php'; // Always use English first
    if (file_exists($filename)) {
        @include($filename);
    }
    if (($language_key = $config['language_key']) != 'en') {
        $filename = dirname(__FILE__).'/languages/'.$language_key.'/aggregator.php';
        if (file_exists($filename)) {
            @include($filename);
        }
    }
    if (empty($string)) {
        $messages[] = 'Internal error: no translations in '.$filename;
        return FALSE;
    }

    // 1A -- prepare for addition of a few nodes
    $parent_id = $config['demo_nodes']['showcase']['node_id'];
    $archive_id = $config['demo_nodes']['newsarchive']['node_id'];
    // place the aggregator-nodes at the end of the parent section
    $sort_order = 0;
    $archive_sort_order = 0;
    foreach($config['demo_nodes'] as $node => $fields) {
        if (($fields['parent_id'] == $parent_id) && ($fields['node_id'] != $parent_id)) {
            $sort_order = max($sort_order,$fields['sort_order']);
        } elseif (($fields['parent_id'] == $archive_id) && ($fields['node_id'] != $archive_id)) {
            $archive_sort_order = max($archive_sort_order,$fields['sort_order']);
        }
    }
    $sort_order += 10;
    $htmlpage_id = intval($config['demo_nodes']['welcome']['module_id']); // 'steal' the module_id
    $nodes = array(
        'aggregator' => array(
            'parent_id'  => $parent_id,
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_link_text'],$config['demo_replace']),
            'sort_order' => $sort_order,
            'module_id'  => $module_id
            ),
        'aggregator_news' => array(
            'parent_id'  => $parent_id,
            'is_page'    => FALSE,
            'title'      => strtr($string['demo_section_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_section_link_text'],$config['demo_replace']),
            'sort_order' => $sort_order + 10
            ),
        'aggregator_news1' => array(
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news1_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news1_link_text'],$config['demo_replace']),
            'sort_order' => 40,
            'module_id'  => $htmlpage_id
            ),
        'aggregator_news2' => array(
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news2_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news2_link_text'],$config['demo_replace']),
            'sort_order' => 30,
            'module_id'  => $htmlpage_id
            ),
        'aggregator_news3' => array(
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news3_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news3_link_text'],$config['demo_replace']),
            'sort_order' => 20,
            'module_id'  => $htmlpage_id
            ),
        'aggregator_news4' => array(
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news4_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news4_link_text'],$config['demo_replace']),
            'sort_order' => 10,
            'module_id'  => $htmlpage_id
            ),
        'archives_news1' => array(
            'parent_id'  => $archive_id,
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news1_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news1_link_text'],$config['demo_replace']),
            'sort_order' => $archive_sort_order + 40,
            'module_id'  => $htmlpage_id
            ),
        'archives_news2' => array(
            'parent_id'  => $archive_id,
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news2_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news2_link_text'],$config['demo_replace']),
            'sort_order' => $archive_sort_order + 30,
            'module_id'  => $htmlpage_id
            ),
        'archives_news3' => array(
            'parent_id'  => $archive_id,
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news3_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news3_link_text'],$config['demo_replace']),
            'sort_order' => $archive_sort_order + 20,
            'module_id'  => $htmlpage_id
            ),
        'archives_news4' => array(
            'parent_id'  => $archive_id,
            'is_page'    => TRUE,
            'title'      => strtr($string['demo_news4_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_news4_link_text'],$config['demo_replace']),
            'sort_order' => $archive_sort_order + 10,
            'module_id'  => $htmlpage_id
            )
        );
    // duplicate the four news pages in the main news section to gain some 'body' there
    $content = array(
        'aggregator_news1' => $string['aggregator_news1_content'],
        'aggregator_news2' => $string['aggregator_news2_content'],
        'aggregator_news3' => $string['aggregator_news3_content'],
        'aggregator_news4' => $string['aggregator_news4_content'],
        'archives_news1' => $string['aggregator_news1_content'],
        'archives_news2' => $string['aggregator_news2_content'],
        'archives_news3' => $string['aggregator_news3_content'],
        'archives_news4' => $string['aggregator_news4_content']
        );

    // 1B -- actually add the necessary nodes & htmlpage content
    $now = strftime('%Y-%m-%d %T');
    $user_id   = $config['user_id'];
    $area_id   = $config['demo_areas']['public']['area_id'];
    foreach($nodes as $node => $fields) {
        if (!isset($fields['parent_id'])) {
            $fields['parent_id'] = intval($config['demo_nodes']['aggregator_news']['node_id']);
        }
        $fields['area_id']  = $area_id;
        $fields['ctime']    = $now;
        $fields['mtime']    = $now;
        $fields['atime']    = $now;
        $fields['owner_id'] = $user_id;
        if (($node_id = db_insert_into_and_get_id('nodes',$fields,'node_id')) === FALSE) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        }
        $node_id = intval($node_id);
        $fields['node_id'] = $node_id;
        $config['demo_nodes'][$node] = $fields;

        if (($fields['is_page']) && ($fields['module_id'] == $htmlpage_id)) {
            $htmlpage_fields = array(
                'node_id' => $node_id,
                'version' => 1,
                'page_data' => strtr($content[$node],$config['demo_replace']),
                'ctime' => $now,
                'cuser_id' => $user_id,
                'mtime' => $now,
                'muser_id' => $user_id);
            if (db_insert_into('htmlpages',$htmlpage_fields) === FALSE) {
                $messages[] = $tr['error'].' '.db_errormessage();
                $retval = FALSE;
            }
        }
    }
    // 2A  -- insert the companion record for our showcase aggregator in aggregator table
    // we want a single snapshots node, and N html-pages from the aggregator_news section
    $nodelist = array();
    if (isset($config['demo_nodes']['snapshots'])) { // snapshots _should_ be installed, but you never know
        $node_list[] = strval($config['demo_nodes']['snapshots']['node_id']);
    }
    $node_list[] = strval($config['demo_nodes']['aggregator_news']['node_id']);

    $fields = array(
            'node_id'            => $config['demo_nodes']['aggregator']['node_id'],
            'header'             => strtr($string['demo_header'],$config['demo_replace']),
            'introduction'       => strtr($string['demo_introduction'],$config['demo_replace']),
            'node_list'          => implode(',',$node_list),
            'items'              => 4,
            'htmlpage_length'    => 2,
            'snapshots_width'    => 512,
            'snapshots_height'   => 120,
            'snapshots_visible'  => 3,
            'snapshots_showtime' => 5,
            'ctime'              => $now,
            'cuser_id'           => $user_id,
            'mtime'              => $now,
            'muser_id'           => $user_id);
    if (db_insert_into('aggregator',$fields) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }

    // 2B -- also insert companion record for existing 'latest news' node
    // we borrow the title of the corresponding node, and point to a single section in node_list
    $latestnews_id = intval($config['demo_nodes']['latestnews']['node_id']);
    $fields['node_id'] = $latestnews_id;
    $fields['header'] = $config['demo_string']['latestnews_title'];
    $fields['introduction'] = strtr($config['demo_string']['latestnews_content'],$config['demo_replace']);
    $fields['node_list'] = strval($archive_id);
    if (db_insert_into('aggregator',$fields) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }

    // 2C -- update the node that was already created in the main demodata as a placeholder
    $fields = array('module_id' => $module_id, 'link_href' => NULL);
    $where = array('node_id' => $latestnews_id);
    if (db_update('nodes',$fields,$where) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }

    // 2D -- move the oldest news to the end of this section, pushing it out of sight in aggregator
    $fields = array('sort_order' => $archive_sort_order + 50);
    $where = array('node_id' => $config['demo_nodes']['oldnews']['node_id']);
    if (db_update('nodes',$fields,$where) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }

    // all done
    return $retval;
} // aggregator_demodata()

?>