<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/althing/althing_install.php - installer of the althing module
 *
 * This file contains the althing module installer.
 * The interface consists of these functions:
 *
 * <code>
 * althing_install(&$messages,$module_id)
 * althing_upgrade(&$messages,$module_id)
 * althing_uninstall(&$messages,$module_id)
 * althing_demodata(&$messages,$module_id,&$config,$manifest)
 * </code>
 *
 * These functions can be called from the main installer and/or admin.php.
 *
 * Note
 * You cannot be sure about the environment from which these routines are called.
 * If the caller is the Install Wizard, you do not have all subroutines available.
 * However, it IS possible to manipulate the database via the db_*() routines and/or
 * the global $DB object. Therefore you have to keep the install routine extremly
 * simple. You also have no option to interact with the user; the install has to
 * be a silent install; you can only indicate success (TRUE) or failure (FALSE)
 * and maybe an error message in $messages[] but that's it. Good luck.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_althing
 * @version $Id: althing_install.php,v 1.16 2016/04/05 08:52:28 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }


/** install the module
 *
 * this routine installs the module. For this module
 * there is nothing to install, so we simply return success.
 *
 * Note that the record for this module is already created in the
 * modules table; the pkey is $module_id.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function althing_install(&$messages,$module_id) {
    return TRUE;
} // althing_install()


/** upgrade the module
 *
 * this routine performs an upgrade to the installed module.
 * Note that the althing module does not need any upgrade at
 * all because there never was an earlier version.
 *
 * For now this routine is a nop.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function althing_upgrade(&$messages,$module_id) {
    return TRUE;
} // althing_upgrade()


/** uninstall the module
 *
 * this is a hook for future extensions of Website@School.
 * For now we simply return success.
 * 
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function althing_uninstall(&$messages,$module_id) {
    return TRUE;
} // althing_uninstall()


/** add demonstration data to the system
 *
 * this routine adds demodata for this module to the database.
 *
 * Note that a routine by this name should exist (even if it does nothing).
 *
 * Note
 * If the module is installed via the Install Wizard, this routine is
 * called. However, if a module is installed as an additional module
 * after installation, the {$module}_demodata() routine is never called.
 * This is because the only time you know that demodata is installed is
 * when the Install Wizard runs. If we're called from admin.php, the
 * webmaster may have already deleted existing (core) demodata so you
 * never can be sure what to expect. To put it another way: it is hard
 * to re-construct $config when we're NOT the Instal Wizard.
 *
 * The array $config contains the following information.
 *
 * <code>
 * $config['language_key']   => install language code (eg. 'en')
 * $config['dir']            => path to CMS Root Directory (eg. /home/httpd/htdocs)
 * $config['www']            => URL of CMS Root Directory (eg. http://exemplum.eu)
 * $config['progdir']        => path to program directory (eg. /home/httpd/htdocs/program)
 * $config['progwww']        => URL of program directory (eg. http://exemplum.eu/program)
 * $config['datadir']        => path to data directory (eg. /home/httpd/wasdata/a1b2c3d4e5f6)
 * $config['user_username']  => userid of webmaster (eg. wblader)
 * $config['user_full_name'] => full name of webmaster (eg. Wilhelmina Bladergroen)
 * $config['user_email']     => email of webmaster (eg. w.bladergroen@exemplum.eu)
 * $config['user_id']        => numerical user_id (usually 1)
 * $config['demo_salt']      => password salt for all demodata accounts
 * $config['demo_password']  => password for all demodata accounts
 * $config['demo_areas']     => array with demo area data
 * $config['demo_groups']    => array with demo group data
 * $config['demo_users']     => array with demo user data
 * $config['demo_nodes']     => array with demo node data
 * $config['demo_string']    => array with demo strings from /program/install/languages/LL/demodata.php
 * $config['demo_replace']   => array with search/replace pairs to 'jazz up' the demo strings
 * </code>
 *
 * With this information, we can add a demonstration configuration for the public area,
 * which shows off the possibilities. Note that we add our own additions to the array
 * $config so other modules and themes can determine the correct status quo w.r.t. the
 * demodata nodes etc.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @param array &$config pertinent data for the new website + demodata foundation
 * @param array $manifest a copy from the manifest for this module
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function althing_demodata(&$messages,$module_id,&$config,$manifest) {
    global $DB;
    $retval = TRUE; // assume success

    // 0 -- get hold of the module-specific translations in $string[]
    $string = array();
    $filename = dirname(__FILE__).'/languages/en/althing.php'; // Always use English first
    if (file_exists($filename)) {
        @include($filename);
    }
    if (($language_key = $config['language_key']) != 'en') {
        $filename = dirname(__FILE__).'/languages/'.$language_key.'/althing.php';
        if (file_exists($filename)) {
            @include($filename);
        }
    }
    if (empty($string)) {
        $messages[] = 'Internal error: no translations in '.$filename;
        return FALSE;
    }

    // 1A -- prepare for addition of a node that will hold the Althing
    // STUB: we really should re-design the demo site so all new modules
    // have a better home than in the News section. Oh well. 2014-05-10
    $sort_order = 0;
    $parent_id = $config['demo_nodes']['showcase']['node_id'];
    foreach($config['demo_nodes'] as $node => $fields) {
        if (($fields['parent_id'] == $parent_id) && ($fields['node_id'] != $parent_id)) {
            $sort_order = max($sort_order,$fields['sort_order']);
        }
    }
    $sort_order += 10; // place the althing-section at the end of the parent section
    $nodes = array(
        'althing' => array(
            'parent_id'  => $parent_id,
            'is_page'    => TRUE,
            'is_default' => FALSE,
            'title'      => strtr($string['demo_node_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_node_link_text'],$config['demo_replace']),
            'sort_order' => $sort_order,
            'module_id'  => $module_id,
            'style'      => '.althing_introduction { background-color: #FFAAFF; }
.althing_question { background-color: #AAFFAA; }
.althing_conclusion { background-color: #FFFFAA; }
.althing_footer { background-color: #AAFFFF; }
.althing_buttonbar { background-color: #FFAAAA; }
.althing_buttonbar_top {
  background-color: #FFAAAA;
  border-bottom: 1px solid #000000;
}
.althing_buttonbar_bottom {
  background-color: #FFAAAA;
  border-bottom: 1px solid #000000;
}
'
            )
        );
    // 1B -- actually add the necessary node
    $now = strftime('%Y-%m-%d %T');
    $user_id   = $config['user_id'];
    $area_id   = $config['demo_areas']['public']['area_id'];
    foreach($nodes as $node => $fields) {
        $fields['area_id']  = $area_id;
        $fields['ctime']    = $now;
        $fields['mtime']    = $now;
        $fields['atime']    = $now;
        $fields['owner_id'] = $user_id;
        if (($node_id = db_insert_into_and_get_id('nodes',$fields,'node_id')) === FALSE) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        }
        $node_id = intval($node_id);
        $fields['node_id'] = $node_id;
        $config['demo_nodes'][$node] = $fields;
    }

    // 1C -- setup permissions for Andrew, Catherine and Maria Montessori via group Seniors
    $group_id = intval($config['demo_groups']['seniors']['group_id']);
    $sql = sprintf('SELECT gc.capacity_code, gc.acl_id, a.permissions_jobs '.
                   'FROM %sgroups_capacities gc JOIN %sacls a '.
                   'USING (acl_id) '.
                   'WHERE gc.group_id = %d',$DB->prefix,$DB->prefix,$group_id);
    if (($DBResult = $DB->query($sql)) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    } elseif (($records = $DBResult->fetch_all_assoc()) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $DBResult->close();
        $retval = FALSE;
    } else {
        $DBResult->close();
        foreach($records as $record) {
            $capacity_code = intval($record['capacity_code']);
            $acl_id = intval($record['acl_id']);
            $permissions_jobs = intval($record['permissions_jobs']);
            if ($capacity_code == 1) {
                $permissions_jobs |= 7;      // Seniors/Pupils: Startcenter+PageManager+FileManager
                $permissions_nodes = 1;      // PageManager: Content Master (for $node_id)
            } elseif ($capacity_code == 2) {
                $permissions_jobs |= 7;      // Seniors/Teachers: Startcenter+PageManager+FileManager
                $permissions_nodes = 15;     // PageManager: Page Master (for $node_id)
            } else {
                $permissions_nodes = 0;
            }
            $where = array('acl_id' => $acl_id);
            $fields = array('permissions_jobs' => $permissions_jobs);
            if (db_update('acls',$fields,$where) === FALSE) {
                $messages[] = $config['demo_string']['error'].db_errormessage();
                $retval = FALSE;
            }
            $fields = array(
                'acl_id' => $acl_id,
                'node_id' => intval($node_id),
                'permissions_modules' => 0,
                'permissions_nodes' => $permissions_nodes);
            if (db_insert_into('acls_nodes',$fields) === FALSE) {
                $messages[] = $config['demo_string']['error'].db_errormessage();
                $retval = FALSE;
            }
        }
    }

    // 2 -- Create an althing on the newly added page at $node_id
    $althing_id = intval($node_id);
    $fields = array(
        'althing_id' => $althing_id,
        'header' => strtr($string['demo_althing_header'],$config['demo_replace']),
        'introduction' => strtr($string['demo_althing_introduction'],$config['demo_replace']),
        'question' => strtr($string['demo_althing_question'],$config['demo_replace']),
        'conclusion' => strtr($string['demo_althing_conclusion'],$config['demo_replace']),
        'footer' => strtr($string['demo_althing_footer'],$config['demo_replace']),
        'status' => 0, // 0=open,1=frozen,2=closed
        'default_visibility' => 1, // 0=hidden, 1=registered, 2=visible
        'email_moderators' => $config['demo_users']['andrew']['email'],
        'email_subscribers' => $config['user_email'],
        'last_display_id' => 12, // is OK when everything works in the next step
        'ctime' => $now,
        'cuser_id' => intval($config['demo_users']['andrew']['user_id']),
        'mtime' => $now,
        'muser_id' => intval($config['demo_users']['andrew']['user_id'])
        );
    if (db_insert_into('althings',$fields) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }

    // 3 -- copy demo files/images from module demodata directory to Helen's data directory
    $pictures = array('{ALTHING_IMG1}' => 'theobroma_small.jpg', '{ALTHING_URL1}' => 'theobroma.jpg',
                      '{ALTHING_IMG2}' => 'droste_small.jpg', '{ALTHING_URL2}' => 'droste.jpg');
    $thumb_prefix = 'zz_thumb_';
    $path_userdir = '/users/'.$config['demo_users']['hparkh']['path']; // relative to $datadir
    $fullpath_source = dirname(__FILE__).'/install/demodata';
    $fullpath_target = $config['datadir'].$path_userdir;
    foreach($pictures as $key => $picture) {
        $filenames = array($picture, $thumb_prefix.$picture);
        foreach ($filenames as $filename) {
            if (!@copy($fullpath_source.'/'.$filename, $fullpath_target.'/'.$filename)) {
                $messages[] = $config['demo_string']['error']."copy('{$path_userdir}/{$filename}')";
                $retval = FALSE;
            }
        }
        $config['demo_replace'][$key] = $path_userdir.'/'.$picture;
    }

    // 4 -- insert a few posts
    $yesterday = strftime('%Y-%m-%d',time()-86400);
    $today = strftime('%Y-%m-%d');
    $demo_posts = array(
        1 => array(
            'author_id' => NULL,
            'author_full_name' => strtr($string['demo_post_full_name1'],$config['demo_replace']),
            'email' => $config['user_email'],
            'subject' => strtr($string['demo_post_subject1'],$config['demo_replace']),
            'datim' => $yesterday.' 09:01:01',
            'content' => strtr($string['demo_post_content1'],$config['demo_replace']),
            'parents' => '0',
            'marbles' => 2
            ),
        2 => array(
            'author_id' => intval($config['demo_users']['catherine']['user_id']),
            'author_full_name' => $config['demo_users']['catherine']['full_name'],
            'email' => $config['demo_users']['catherine']['email'],
            'subject' => strtr($string['demo_post_subject2'],$config['demo_replace']),
            'datim' => $yesterday.' 10:12:02',
            'content' => strtr($string['demo_post_content2'],$config['demo_replace']),
            'parents' => '1'
            ),
        3 => array(
            'author_id' => intval($config['demo_users']['hparkh']['user_id']),
            'author_full_name' => $config['demo_users']['hparkh']['full_name'],
            'email' => $config['demo_users']['hparkh']['email'],
            'subject' => strtr($string['demo_post_subject3'],$config['demo_replace']),
            'datim' => $today.' 09:23:03',
            'content' => strtr($string['demo_post_content3'],$config['demo_replace']),
            'parents' => '0'
            ),
        4 => array(
            'author_id' => NULL,
            'author_full_name' => strtr($string['demo_post_full_name4'],$config['demo_replace']),
            'email' => $config['user_email'],
            'subject' => strtr($string['demo_post_subject4'],$config['demo_replace']),
            'datim' => $today.' 12:34:04',
            'content' => strtr($string['demo_post_content4'],$config['demo_replace']),
            'parents' => '0'
            ),
        5 => array(
            'author_id' => intval($config['demo_users']['andrew']['user_id']),
            'author_full_name' => $config['demo_users']['andrew']['full_name'],
            'email' => $config['demo_users']['andrew']['email'],
            'subject' => strtr($string['demo_post_subject5'],$config['demo_replace']),
            'datim' => $today.' 16:11:05',
            'content' => strtr($string['demo_post_content5'],$config['demo_replace']),
            'parents' => '4'
            ),
        6 => array(
            'author_id' => intval($config['demo_users']['catherine']['user_id']),
            'author_full_name' => $config['demo_users']['catherine']['full_name'],
            'email' => $config['demo_users']['catherine']['email'],
            'subject' => strtr($string['demo_post_subject6'],$config['demo_replace']),
            'datim' => $today.' 17:57:06',
            'content' => strtr($string['demo_post_content6'],$config['demo_replace']),
            'parents' => '5'
            ),
        7 => array(
            'author_id' => intval($config['demo_users']['georgina']['user_id']),
            'author_full_name' => $config['demo_users']['georgina']['full_name'],
            'email' => $config['demo_users']['georgina']['email'],
            'subject' => strtr($string['demo_post_subject7'],$config['demo_replace']),
            'datim' => $today.' 18:01:07',
            'content' => strtr($string['demo_post_content7'],$config['demo_replace']),
            'parents' => '5'
            ),
        8 => array(
            'author_id' => intval($config['demo_users']['hparkh']['user_id']),
            'author_full_name' => $config['demo_users']['hparkh']['full_name'],
            'email' => $config['demo_users']['hparkh']['email'],
            'subject' => strtr($string['demo_post_subject8'],$config['demo_replace']),
            'datim' => $today.' 18:23:08',
            'content' => strtr($string['demo_post_content8'],$config['demo_replace']),
            'parents' => '0'
            ),
        9 => array(
            'author_id' => intval($config['demo_users']['andrew']['user_id']),
            'author_full_name' => $config['demo_users']['andrew']['full_name'],
            'email' => $config['demo_users']['andrew']['email'],
            'subject' => strtr($string['demo_post_subject9'],$config['demo_replace']),
            'datim' => $today.' 19:17:09',
            'content' => strtr($string['demo_post_content9'],$config['demo_replace']),
            'parents' => '8'
            ),
        10 => array(
            'author_id' => NULL,
            'author_full_name' => strtr($string['demo_post_full_name10'],$config['demo_replace']),
            'email' => $config['user_email'],
            'subject' => strtr($string['demo_post_subject10'],$config['demo_replace']),
            'datim' => $today.' 20:51:10',
            'content' => strtr($string['demo_post_content10'],$config['demo_replace']),
            'parents' => '0'
            ),
        11 => array(
            'author_id' => intval($config['demo_users']['mmonte']['user_id']),
            'author_full_name' => $config['demo_users']['mmonte']['full_name'],
            'email' => $config['demo_users']['mmonte']['email'],
            'subject' => strtr($string['demo_post_subject11'],$config['demo_replace']),
            'datim' => $today.' 23:39:11',
            'content' => strtr($string['demo_post_content11'],$config['demo_replace']),
            'parents' => '5,6,7'
            ),
        12 => array(
            'author_id' => NULL,
            'author_full_name' => strtr($string['demo_post_full_name10'],$config['demo_replace']),
            'email' => $config['user_email'],
            'subject' => strtr($string['demo_post_subject12'],$config['demo_replace']),
            'datim' => $today.' 23:59:12',
            'content' => strtr($string['demo_post_content12'],$config['demo_replace']),
            'parents' => '0'
            )
        );
    foreach($demo_posts as $display_id => $demo_post) {
        // show off the addition of marbles on post 1 by Catherine,
        // show off a moderated post #10 (published initially but hidden afterwards by mmonte)
        // and an unpublished and hidden post #12 with an angry reaction by AC4
        $visible = TRUE;
        $published = TRUE;
        $mtime = $demo_post['datim'];
        $muser_id = $demo_post['author_id'];
        if ($display_id == 1) {
            $mtime = $yesterday.' 10:10:02';
            $muser_id = intval($config['demo_users']['catherine']['user_id']);
        } elseif ($display_id == 10) {
            $visible = FALSE;
            $mtime = $today.' 23:43:11';
            $muser_id = intval($config['demo_users']['mmonte']['user_id']);
        } elseif ($display_id == 12) {
            $visible = FALSE;
            $published = FALSE;
        }
        $fields = array(
            'althing_id' => $althing_id,
            'display_id' => intval($display_id),
            'author_id' => $demo_post['author_id'],
            'author_full_name' => $demo_post['author_full_name'],
            'email' => $demo_post['email'],
            'subject' => $demo_post['subject'],
            'datim' => $demo_post['datim'],
            'content' => $demo_post['content'],
            'visible' => $visible,
            'published' => $published,
            'marbles' => (isset($demo_post['marbles'])) ? intval($demo_post['marbles']) : 0,
            'remote_addr' => '127.0.0.1',
            'mtime' => $mtime,
            'muser_id' => $muser_id
            );
        if (($post_id = db_insert_into_and_get_id('althing_posts',$fields,'post_id')) === FALSE) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        } else {
            $demo_posts[$display_id]['post_id'] = intval($post_id);
        }
    }
    // 5 -- and add the interconnections between the posts now we have the post_id's
    foreach($demo_posts as $child_display_id => $demo_post) {
        if ($demo_post['parents'] != '0') {
            $parents = explode(',',$demo_post['parents']);
            foreach ($parents as $parent_display_id) {
                $fields = array(
                    'parent_id'  => intval($demo_posts[$parent_display_id]['post_id']),
                    'child_id'   => intval($demo_posts[$child_display_id]['post_id']),
                    'althing_id' => $althing_id
                    );
                if (db_insert_into('althing_relations',$fields) === FALSE) {
                    $messages[] = $config['demo_string']['error'].db_errormessage();
                    $retval = FALSE;
                }
            }
        }
    }
    // 6 -- add 2 log messages to show off moderation logging
    $aparams = array(
        '{OLD_VISIBILITY}' => $string['moderation_action_visible'],
        '{NEW_VISIBILITY}' => $string['moderation_action_visible'],
        '{OLD_MARBLES}' => '0',
        '{NEW_MARBLES}' => '2');
    $action = strtr($string['moderation_action_marbles'],$aparams);
    $fields = array(
        'post_id' => intval($demo_posts[1]['post_id']),
        'althing_id' => $althing_id,
        'action' => $action,
        'remark' => $string['demo_log_remark1'],
        'mtime' => $yesterday.' 10:10:02', // same as $mtime in post #1 above
        'muser_id' => intval($config['demo_users']['catherine']['user_id'])
        );
    if (db_insert_into('althing_log_messages',$fields) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }
    $aparams = array(
        '{OLD_VISIBILITY}' => $string['moderation_action_visible'],
        '{NEW_VISIBILITY}' => $string['moderation_action_hidden'],
        '{OLD_MARBLES}' => '0',
        '{NEW_MARBLES}' => '0');
    $action = strtr($string['moderation_action_visibility'],$aparams);
    $fields = array(
        'post_id' => intval($demo_posts[10]['post_id']),
        'althing_id' => $althing_id,
        'action' => $action,
        'remark' => $string['demo_log_remark10'],
        'mtime' => $today.' 23:43:11', // same as $mtime in post #10 above
        'muser_id' => intval($config['demo_users']['mmonte']['user_id'])
        );
    if (db_insert_into('althing_log_messages',$fields) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }
    // 7 -- all done
    return $retval;
} // althing_demodata()

?>