<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/althing/install/althing_tabledefs.php - data definition for module
 *
 * A total of 4 tables are defined:
 *
 * althings             - 1 record per althing, tied to a node; contains main configuration
 * althing_posts        - 0, 1 or more records containing visitor comments to this althing
 * althing_relations    - 0, 1 or more records store the parent-child relations between posts
 * althing_log_messages - 0, 1 or more moderator messages/log messages
 *
 * The tables 'althings' and 'althing_posts' and have straightforward
 * 1-to-N relations. The 'althing_relations' table stores the connections
 * between messages. This requires a separate table because of the special
 * feature of an althing that allows for a reply to more than one post at 
 * the same time. We need to keep track of the multiple parents in a
 * convenient way. The table 'althing_log_messages' keeps track of all
 * moderation events with structured descriptions and optional moderator remarks.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_althing
 * @version $Id: althing_tabledefs.php,v 1.6 2016/03/23 09:28:21 peter Exp $
 */
$tabledefs['althings'] = array(
    'name' => 'althings',
    'comment' => 'main table for althing module stores the configuration data',
    'fields' => array(
        array(
            'name' => 'althing_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the node this althing is connected to'
            ),
        array(
            'name' => 'header',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'the (optional) title to display at the top of the page'
            ),
        array(
            'name' => 'introduction',
            'type' => 'text',
            'comment' => 'the (optional) introduction/explanation above the page'
            ),
        array(
            'name' => 'question',
            'type' => 'text',
            'comment' => 'the (optional) research question of this althing'
            ),
        array(
            'name' => 'conclusion',
            'type' => 'text',
            'comment' => 'the (optional) outcome of the research in this althing'
            ),
        array(
            'name' => 'footer',
            'type' => 'text',
            'comment' => 'the (optional) footer at the bottom of the althing'
            ),
        array(
            'name' => 'status',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'current mode of the althing: 0=open, 1=frozen, 2=closed'
            ),
        array(
            'name' => 'default_visibility',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'new posts visibility: 0=hidden, 1=registered, 2=visible'
            ),
        array(
            'name' => 'email_moderators',
            'type' => 'text',
            'comment' => 'comma or newline-delimited list of moderator email addresses'
            ),
        array(
            'name' => 'email_subscribers',
            'type' => 'text',
            'comment' => 'comma or newline-delimited list of subscriber email addresses'
            ),
        array(
            'name' => 'ctime',
            'type' => 'datetime',
            'comment' => 'contains the time the althing was created'
            ),
        array(
            'name' => 'cuser_id',
            'type' => 'int',
            'comment' => 'identifies the user that created this althing'
            ),
        array(
            'name' => 'mtime',
            'type' => 'datetime',
            'comment' => 'contains the time the configuration was last updated'
            ),
        array(
            'name' => 'muser_id',
            'type' => 'int',
            'comment' => 'identifies the user that last modified the configuration'
            ),
        array(
            'name' => 'last_display_id',
            'type' => 'int',
            'default' => 0,
            'comment' => 'provides unique user-friendly display_id for posts'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('althing_id')
            ),
        array(
            'name' => 'node',
            'type' => 'foreign',
            'fields' => array('althing_id'),
            'reftable' => 'nodes',
            'reffields' => array('node_id')
            ),
        array(
            'name' => 'cuser',
            'type' => 'foreign',
            'fields' => array('cuser_id'),
            'reftable' => 'users',
            'reffields' => array('user_id')
            ),
        array(
            'name' => 'muser',
            'type' => 'foreign',
            'fields' => array('muser_id'),
            'reftable' => 'users',
            'reffields' => array('user_id')
            )
        )
    );
$tabledefs['althing_posts'] = array(
    'name' => 'althing_posts',
    'comment' => 'stores all visitor posts',
    'fields' => array(
        array(
            'name' => 'post_id',
            'type' => 'serial',
            'comment' => 'unique identification of an althing post'
            ),
        array(
            'name' => 'althing_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the althing to which this post belongs'
            ),
        array(
            'name' => 'display_id',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => 'unique number identifying posts within this althing starting with 1'
            ),
        array(
            'name' => 'author_id',
            'type' => 'int',
            'notnull' => FALSE,
            'default' => 'NULL',
            'comment' => 'identifies the registered user that replied, NULL=anonymous coward'
            ),
        array(
            'name' => 'author_full_name',
            'type' => 'varchar',
            'length' => 255,
            'comment' => 'registered user\'s full name OR a user-defined nickname'
            ),
        array(
            'name' => 'email',
            'type' => 'varchar',
            'length' => 255,
            'comment' => 'registered user\'s e-mail address OR a user-defined e-mail address'
            ),
        array(
            'name' => 'subject',
            'type' => 'varchar',
            'length' => 80,
            'comment' => '(optional) subject of the post'
            ),
        array(
            'name' => 'datim',
            'type' => 'datetime',
            'comment' => 'contains the date/time the post was submitted'
            ),
        array(
            'name' => 'content',
            'type' => 'text',
            'comment' => 'the actual content of the reply'
            ),
        array(
            'name' => 'visible',
            'type' => 'bool',
            'notnull' => TRUE,
            'default' => FALSE,
            'comment' => 'TRUE=show this post, FALSE=suppressed/invisible'
            ),
        array(
            'name' => 'published',
            'type' => 'bool',
            'notnull' => TRUE,
            'default' => FALSE,
            'comment' => 'TRUE=this post is/has been visible, FALSE=new, invisible post'
            ),
        array(
            'name' => 'marbles',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => 'a good post can earn credits (based on moderators discretion)'
            ),
        array(
            'name' => 'remote_addr',
            'type' => 'varchar',
            'length' => 150,
            'notnull' => TRUE,
            'comment' => 'IP-address of the visitor'
            ),
        array(
            'name' => 'mtime',
            'type' => 'datetime',
            'comment' => 'contains the time the post was last moderated'
            ),
        array(
            'name' => 'muser_id',
            'type' => 'int',
            'comment' => 'identifies the user that last moderated this post'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('post_id')
            ),
        array(
            'name' => 'althing',
            'type' => 'foreign',
            'fields' => array('althing_id'),
            'reftable' => 'althings',
            'reffields' => array('althing_id')
            ),
        array(
            'name' => 'muser',
            'type' => 'foreign',
            'fields' => array('muser_id'),
            'reftable' => 'users',
            'reffields' => array('user_id')
            ),
        array(
            'name' => 'althing_display',
            'type' => 'index',
            'unique' => TRUE,
            'fields' => array('althing_id','display_id'),
            'comment' => 'enforce unique display_id within an althing'
            ),
        array(
            'name' => 'althing_index',
            'type' => 'index',
            'fields' => array('althing_id','post_id'),
            'comment' => 'hint for quick selections based on althing_id'
            )
        )
    );
$tabledefs['althing_relations'] = array(
    'name' => 'althing_relations',
    'comment' => 'stores all relations between visitor posts',
    'fields' => array(
        array(
            'name' => 'parent_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'identifies the post to which the child is a reply'
            ),
        array(
            'name' => 'child_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'identifies the post that is a reply to the parent post'
            ),
        array(
            'name' => 'althing_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the althing to which this relation belongs' // superfluous but handy for delete
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('parent_id','child_id')
            ),
        array(
            'name' => 'parent',
            'type' => 'foreign',
            'fields' => array('parent_id'),
            'reftable' => 'althing_posts',
            'reffields' => array('post_id')
            ),
        array(
            'name' => 'child',
            'type' => 'foreign',
            'fields' => array('child_id'),
            'reftable' => 'althing_posts',
            'reffields' => array('post_id')
            ),
        array(
            'name' => 'althing',
            'type' => 'foreign',
            'fields' => array('althing_id'),
            'reftable' => 'althings',
            'reffields' => array('althing_id')
            ),
        array(
            'name' => 'child_parent_index',
            'type' => 'index',
            'fields' => array('child_id','parent_id'),
            'comment' => 'hint for quick reverse selections'
            )
        )
    );
$tabledefs['althing_log_messages'] = array(
    'name' => 'althing_log_messages',
    'comment' => 'stores all moderation events to visitor posts',
    'fields' => array(
        array(
            'name' => 'log_message_id',
            'type' => 'serial',
            'comment' => 'unique identification of an althing moderation event'
            ),
        array(
            'name' => 'post_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the althing post to which this moderation belongs'
            ),
        array(
            'name' => 'althing_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the althing to which this moderation belongs' // superfluous but handy for delete
            ),
        array(
            'name' => 'action',
            'type' => 'text',
            'comment' => 'the generated message indicating the action that took place'
            ),
        array(
            'name' => 'remark',
            'type' => 'text',
            'comment' => 'the free-format message allowing moderator meta-talk'
            ),
        array(
            'name' => 'mtime',
            'type' => 'datetime',
            'comment' => 'contains the time the post was last moderated'
            ),
        array(
            'name' => 'muser_id',
            'type' => 'int',
            'comment' => 'identifies the user that last moderated this post'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('log_message_id')
            ),
        array(
            'name' => 'post',
            'type' => 'foreign',
            'fields' => array('post_id'),
            'reftable' => 'althing_posts',
            'reffields' => array('post_id')
            ),
        array(
            'name' => 'althing',
            'type' => 'foreign',
            'fields' => array('althing_id'),
            'reftable' => 'althings',
            'reffields' => array('althing_id')
            ),
        array(
            'name' => 'muser',
            'type' => 'foreign',
            'fields' => array('muser_id'),
            'reftable' => 'users',
            'reffields' => array('user_id')
            )
        )
    );


?>