<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/confab/confab_install.php - installer of the confab module
 *
 * This file contains the confab module installer.
 * The interface consists of these functions:
 *
 * <code>
 * confab_install(&$messages,$module_id)
 * confab_upgrade(&$messages,$module_id)
 * confab_uninstall(&$messages,$module_id)
 * confab_demodata(&$messages,$module_id,&$config,$manifest)
 * </code>
 *
 * These functions can be called from the main installer and/or admin.php.
 *
 * Note
 * You cannot be sure about the environment from which these routines are called.
 * If the caller is the Install Wizard, you do not have all subroutines available.
 * However, it IS possible to manipulate the database via the db_*() routines and/or
 * the global $DB object. Therefore you have to keep the install routine extremly
 * simple. You also have no option to interact with the user; the install has to
 * be a silent install; you can only indicate success (TRUE) or failure (FALSE)
 * and maybe an error message in $messages[] but that's it. Good luck.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_confab
 * @version $Id: confab_install.php,v 1.8 2016/03/23 09:28:21 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }


/** install the module
 *
 * this routine installs the module. For this module
 * there is nothing to install, so we simply return success.
 *
 * Note that the record for this module is already created in the
 * modules table; the pkey is $module_id.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function confab_install(&$messages,$module_id) {
    return TRUE;
} // confab_install()


/** upgrade the module
 *
 * this routine performs an upgrade to the installed module.
 * Note that the confab module does not need any upgrade at
 * all because there never was an earlier version.
 *
 * For now this routine is a nop.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function confab_upgrade(&$messages,$module_id) {
    return TRUE;
} // confab_upgrade()


/** uninstall the module
 *
 * this is a hook for future extensions of Website@School.
 * For now we simply return success.
 * 
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function confab_uninstall(&$messages,$module_id) {
    return TRUE;
} // confab_uninstall()


/** add demonstration data to the system
 *
 * this routine adds demodata for this module to the database.
 *
 * Note that a routine by this name should exist (even if it does nothing).
 *
 * Note
 * If the module is installed via the Install Wizard, this routine is
 * called. However, if a module is installed as an additional module
 * after installation, the {$module}_demodata() routine is never called.
 * This is because the only time you know that demodata is installed is
 * when the Install Wizard runs. If we're called from admin.php, the
 * webmaster may have already deleted existing (core) demodata so you
 * never can be sure what to expect. To put it another way: it is hard
 * to re-construct $config when we're NOT the Instal Wizard.
 *
 * The array $config contains the following information.
 *
 * <code>
 * $config['language_key']   => install language code (eg. 'en')
 * $config['dir']            => path to CMS Root Directory (eg. /home/httpd/htdocs)
 * $config['www']            => URL of CMS Root Directory (eg. http://exemplum.eu)
 * $config['progdir']        => path to program directory (eg. /home/httpd/htdocs/program)
 * $config['progwww']        => URL of program directory (eg. http://exemplum.eu/program)
 * $config['datadir']        => path to data directory (eg. /home/httpd/wasdata/a1b2c3d4e5f6)
 * $config['user_username']  => userid of webmaster (eg. wblader)
 * $config['user_full_name'] => full name of webmaster (eg. Wilhelmina Bladergroen)
 * $config['user_email']     => email of webmaster (eg. w.bladergroen@exemplum.eu)
 * $config['user_id']        => numerical user_id (usually 1)
 * $config['demo_salt']      => password salt for all demodata accounts
 * $config['demo_password']  => password for all demodata accounts
 * $config['demo_areas']     => array with demo area data
 * $config['demo_groups']    => array with demo group data
 * $config['demo_users']     => array with demo user data
 * $config['demo_nodes']     => array with demo node data
 * $config['demo_string']    => array with demo strings from /program/install/languages/LL/demodata.php
 * $config['demo_replace']   => array with search/replace pairs to 'jazz up' the demo strings
 * </code>
 *
 * With this information, we can add a demonstration configuration for the public area,
 * which shows off the possibilities. Note that we add our own additions to the array
 * $config so other modules and themes can determine the correct status quo w.r.t. the
 * demodata nodes etc.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @param array &$config pertinent data for the new website + demodata foundation
 * @param array $manifest a copy from the manifest for this module
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function confab_demodata(&$messages,$module_id,&$config,$manifest) {
    global $DB;
    $retval = TRUE; // assume success

    // 0 -- get hold of the module-specific translations in $string[]
    $string = array();
    $filename = dirname(__FILE__).'/languages/en/confab.php'; // Always use English first
    if (file_exists($filename)) {
        @include($filename);
    }
    if (($language_key = $config['language_key']) != 'en') {
        $filename = dirname(__FILE__).'/languages/'.$language_key.'/confab.php';
        if (file_exists($filename)) {
            @include($filename);
        }
    }
    if (empty($string)) {
        $messages[] = 'Internal error: no translations in '.$filename;
        return FALSE;
    }

    // 1A -- prepare for addition of a node that will hold the Confab (goes into new showcase section)
    $sort_order = 0;
    $parent_id = $config['demo_nodes']['showcase']['node_id'];
    foreach($config['demo_nodes'] as $node => $fields) {
        if (($fields['parent_id'] == $parent_id) && ($fields['node_id'] != $parent_id)) {
            $sort_order = max($sort_order,$fields['sort_order']);
        }
    }
    $sort_order += 10; // place the confab at the end of the parent section, for now
    $nodes = array(
        'confab' => array(
            'parent_id'  => $parent_id,
            'is_page'    => TRUE,
            'is_default' => FALSE,
            'title'      => strtr($string['demo_node_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_node_link_text'],$config['demo_replace']),
            'sort_order' => $sort_order,
            'module_id'  => $module_id,
            'style'      => '.confab_introduction { background-color: #FFAAAA; }
.confab_footer { background-color: #AAAAFF; }
'
            )
        );
    // 1B -- actually add the necessary node
    $now = strftime('%Y-%m-%d %T');
    $user_id   = $config['user_id'];
    $area_id   = $config['demo_areas']['public']['area_id'];
    foreach($nodes as $node => $fields) {
        $fields['area_id']  = $area_id;
        $fields['ctime']    = $now;
        $fields['mtime']    = $now;
        $fields['atime']    = $now;
        $fields['owner_id'] = $user_id;
        if (($node_id = db_insert_into_and_get_id('nodes',$fields,'node_id')) === FALSE) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        }
        $node_id = intval($node_id);
        $fields['node_id'] = $node_id;
        $config['demo_nodes'][$node] = $fields;
    }

    // 1C -- setup perms for Andrew, Catherine (none) and Maria Montessori (Page Master) via group Seniors
    $group_id = intval($config['demo_groups']['seniors']['group_id']);
    $sql = sprintf('SELECT gc.capacity_code, gc.acl_id, a.permissions_jobs '.
                   'FROM %sgroups_capacities gc JOIN %sacls a '.
                   'USING (acl_id) '.
                   'WHERE gc.group_id = %d',$DB->prefix,$DB->prefix,$group_id);
    if (($DBResult = $DB->query($sql)) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    } elseif (($records = $DBResult->fetch_all_assoc()) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $DBResult->close();
        $retval = FALSE;
    } else {
        $DBResult->close();
        // 1C1 -- we expect 2 $records, one for the pupils and one for the teachers in the group
        foreach($records as $record) {
            $capacity_code = intval($record['capacity_code']);
            $acl_id = intval($record['acl_id']);
            $permissions_jobs = intval($record['permissions_jobs']);
            if ($capacity_code == 1) {
                continue;                    // Seniors/Pupils get no access to this Confab page at all
            } elseif ($capacity_code == 2) {
                $permissions_jobs |= 7;      // Seniors/Teachers: Startcenter+PageManager+FileManager
                $permissions_nodes = 15;     // PageManager: Page Master (for $node_id)
            } else {
                $permissions_nodes = 0;      // shouldn't happen
            }
            // 1C2 -- update job permissions for Seniors/Teachers
            $where = array('acl_id' => $acl_id);
            $fields = array('permissions_jobs' => $permissions_jobs);
            if (db_update('acls',$fields,$where) === FALSE) {
                $messages[] = $config['demo_string']['error'].db_errormessage();
                $retval = FALSE;
            }
            // 1C3 -- insert a new ACL node record with perms for $node_id we just created in step 1B
            $fields = array(
                'acl_id' => $acl_id,
                'node_id' => intval($node_id),
                'permissions_modules' => 0,
                'permissions_nodes' => $permissions_nodes);
            if (db_insert_into('acls_nodes',$fields) === FALSE) {
                $messages[] = $config['demo_string']['error'].db_errormessage();
                $retval = FALSE;
            }
        }
    }

    // 2A -- Create a confab on the newly added page at $node_id
    $confab_id = intval($node_id);
    $config['demo_replace']['{CONFAB_GROUP_FULL_NAME}'] = $config['demo_groups']['seniors']['full_name'];
    $config['demo_replace']['{CONFAB_TEACHER_FULL_NAME}'] = $config['demo_users']['mmonte']['full_name'];
    $fields = array(
        'confab_id' => $confab_id,
        'header' => strtr($string['demo_confab_header'],$config['demo_replace']),
        'introduction' => strtr($string['demo_confab_introduction'],$config['demo_replace']),
        'footer' => strtr($string['demo_confab_footer'],$config['demo_replace']),
        'passcode' => trim(strtr($string['demo_confab_passcode'],$config['demo_replace'])),
        'max_participants' => 6,        // it is only a small school
        'max_tries' => 10,
        'refresh' => 6,
        'timeout_idle' => 300,          // 5 minutes after last message from the participant
        'timeout_conversation' => 3600, // 1 hour after last message from any participant
        'status' => 1,                  // CONFAB_STATUS_REGISTERED
        'personal' => 1,                // CONFAB_PERSONAL_REGISTERED
        'last_conversation_id' => 2,    // 1=Andrew in the morning, 2=Andrew+Catherine in the afternoon
        'last_participant_id' => 2,     // Andrew=1, Catherine=2, Maria=system/moderator=0
        'ctime' => $now,
        'cuser_id' => intval($config['demo_users']['mmonte']['user_id']),
        'mtime' => $now,
        'muser_id' => intval($config['demo_users']['mmonte']['user_id'])
        );
    if (db_insert_into('confabs',$fields) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }
    // 2B -- Create the first two conversations in this confab
    $andrew    = $config['demo_users']['andrew']; $andrew['ip'] = '192.0.2.65';
    $catherine = $config['demo_users']['catherine']; $catherine['ip'] = '192.0.2.67';
    $maria     = $config['demo_users']['mmonte']; $maria['ip'] = '192.0.2.77';
    $hello_catherine = strtr($string['demo_hello'],array('{USERNAME}' => $catherine['username']));
    $hello_andrew = strtr($string['demo_hello'],array('{USERNAME}' => $andrew['username']));
    $lines = array(
        //    C  SYS.  PERS.  TIME        Id From        To       Message
        array(1, TRUE, FALSE, '08:45:16', 1, $andrew,    NULL,    $string['join_full']),
        array(1, FALSE,FALSE, '08:45:23', 1, $andrew,    NULL,    $string['demo_anybody']),
        array(1, FALSE,FALSE, '08:45:31', 1, $andrew,    NULL,    $string['demo_anybody']),
        array(1, FALSE,FALSE, '08:45:47', 1, $andrew,    NULL,    $string['demo_anybody']),
        array(1, FALSE,FALSE, '08:46:17', 1, $andrew,    NULL,    $string['demo_anybody']),
        array(1, FALSE,FALSE, '08:46:56', 1, $andrew,    NULL,    $string['demo_nobody']),
        array(1, TRUE, FALSE, '08:47:03', 1, $andrew,    NULL,    $string['leave_full']),
        array(2, TRUE, FALSE, '15:30:53', 1, $andrew,    NULL,    $string['join_full']),
        array(2, TRUE, FALSE, '15:31:13', 2, $catherine, NULL,    $string['join_full']),
        array(2, FALSE,FALSE, '15:31:21', 1, $andrew,    NULL,    $hello_catherine),
        array(2, FALSE,FALSE, '15:31:23', 1, $andrew,    NULL,    $string['demo_homework']),
        array(2, FALSE,FALSE, '15:31:41', 2, $catherine, NULL,    $hello_andrew),
        array(2, FALSE,FALSE, '15:32:19', 2, $catherine, NULL,    $string['demo_waitasec']),
        array(2, FALSE,FALSE, '15:33:11', 2, $catherine, NULL,    $string['demo_summary']),
        array(2, FALSE,FALSE, '15:33:23', 1, $andrew,    NULL,    $string['demo_ok']),
        array(2, FALSE,FALSE, '15:33:35', 1, $andrew,    NULL,    $string['demo_sendme']),
        array(2, FALSE,FALSE, '15:33:39', 1, $andrew,    NULL,    $string['demo_please']),
        array(2, FALSE,TRUE,  '15:33:50', 0, $maria,     $andrew, $string['demo_ownwork']),
        array(2, FALSE,FALSE, '15:34:01', 1, $andrew,    NULL,    $string['demo_nevermind']),
        array(2, TRUE, FALSE, '15:34:03', 1, $andrew,    NULL,    $string['leave_full']),
        array(2, FALSE,FALSE, '15:34:15', 2, $catherine, NULL,    $string['demo_bye']),
        array(2, TRUE, FALSE, '15:34:18', 2, $catherine, NULL,    $string['leave_full'])
        );
    // our demo-conversations must take place yesterday, because otherwise
    // an unsuspecting  user might continue with the existing conversation #2 whereas we want
    // a new participant start a new conversation. This doesn't really match the $now
    // value in the node record. Oh well.
    $yesterday = strftime('%Y-%m-%d ',time() - 86400);
    foreach($lines as $line) {
        $from_to = array(
            '{FROM}'          => $line[5]['username'],
            '{FROM_NAME}'     => $line[5]['full_name'],
            '{TO}'            => ($line[2]) ? $line[6]['username'] : '',
            '{TO_NAME}'       => ($line[2]) ? $line[6]['full_name'] : ''
            );
        $fields = array(
            'confab_id'        => $confab_id,
            'conversation_id'  => $line[0],
            'message_system'   => $line[1],
            'message_personal' => $line[2],
            'message_datim'    => $yesterday.$line[3],
            'message_from_id'  => $line[4],
            'message_from'     => $line[5]['username'],
            'message_to'       => ($line[2]) ? $line[6]['username'] : NULL,
            'message'          => strtr($line[7],$from_to),
            'remote_addr'      => $line[5]['ip']
            );
        if (db_insert_into('confab_messages',$fields) === FALSE) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        }
    }

    // 3 -- all done
    return $retval;
} // confab_demodata()

?>