<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/confab/confab_search.php - interface to the search-part of this module
 *
 * This file defines the interface with this module for searching content.
 * The interface consists of a (temporary) table and this function:
 *
 * <code>
 * bool confab_search(&$hits,&$results,$qwords, $limit, $offset)
 * </code>
 *
 * The function is called whenever module content is to be searched. At that point
 * the helper table, which is called 'search_nodes', exists and is populated.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_confab
 * @version $Id: confab_search.php,v 1.4 2016/04/05 14:33:01 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }


/** search this module's content in selected nodes for keywords in $qwords 
 *
 * This interface is documented in {@link htmlpage_search()} in {@link htmlpage_search.php}.
 *
 * this module requires an extra where clause to select only those confabs that
 * are open for registered users (status = 1) or open for anyone (status = 2).
 * this does NOT mean that conversations can be searched (they can't) but either
 * way the header/introduction/footer can be searched. Only if status=0 there
 * cannot be any results because there is just a message 'this confab closed'.
 *
 * @param array &$hits receives search results (at most $limit)
 * @param int &$results receives number of hits (negative value on entry means (re-)calculate total) 
 * @param array $qwords holds 1 or more keywords to search for (original, utf8lower, quoted)
 * @param int $limit maximum number of hits to return (could be 0)
 * @param int $offset starting point within this module for returning results
 * @return bool|array FALSE on failure, TRUE on success and $hits, $results updated (maybe)
 */
function confab_search(&$hits, &$results, $qwords, $limit, $offset=0) {
    global $DB;

    // 1 -- anything to do?
    if (($results == 0) || (($results > 0) && ($limit <= 0))) {
        return TRUE;
    }

    // 2 -- prepare the selection criteria etc.
    $table = 'confabs';
    $fields = array('header' => 'm.header', 'introduction' => 'm.introduction', 'footer' => 'm.footer');
    $where = sprintf('((status = 1) OR (status = 2)) AND %s',search_where($qwords,$fields));
    $order = 'n.node_id';
    $node = 'confab_id';
    $ttl = 'header';

    // 3 -- perform actual search + return answers to caller
    return search_simple($hits,$results,$qwords,$limit,$offset,$table,$fields,$where,$order,$node,$ttl);
} // confab_search()

?>