<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/confab/install/confab_tabledefs.php - data definition for module
 *
 * A total of 3 tables is defined:
 *
 * confabs             - 1 record per confab, tied to a node; contains main configuration
 * confab_messages     - 0, 1 or more records containing visitor contributions to conversations
 * confab_participants - 0, 1 or more records containing transient visitor information
 *
 * The tables 'confabs' and 'confab_messages' have a relative straightforward
 * 1-to-N relation. A record in confab_participants is created once a visitor joins
 * the conversation (and presenting the correct pass code). The record only exists
 * while the visitor is engaged in the conversation. Once a visitor leaves the
 * conversation, the corresponding record is deleted. A conversation is completely
 * self-contained in the confab_messages table.
 *
 * Note:
 * The length of the username in the table confab_participants below is set at 64.
 * A system username is limited to a length of 60. This allows for at most
 * 4 characters to distinguish Anonymous and Registered participants in the
 * participants and messages tables. Feels like a kludge. Sorry. But wait, it gets
 * worse...
 * Because a registered user cannot change her username on joining the conversation,
 * we have to have a way to distinguish multiple instances of the same USER->username.
 * The best I could think of was to add a numeric 'username extension' that counts from
 * 1 to N. Also, I now make a distinction between the username, ie. what the user
 * entered in the dialog and the nickname which is used in the messages table.
 * For anonymous users the nickname is the username with a postfix. For registered
 * users it is their USER->username, with a postfix '(nnn)' for the second and following
 * instances. Ugly. The extra space is 12: enough for '(' and 10 digits and ')'. A
 * positive 32-bit number fits in that space...
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_confab
 * @version $Id: confab_tabledefs.php,v 1.11 2016/03/23 09:28:21 peter Exp $
 */
$tabledefs['confabs'] = array(
    'name' => 'confabs',
    'comment' => 'main table for confab module stores the configuration data',
    'fields' => array(
        array(
            'name' => 'confab_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the node this confab is connected to'
            ),
        array(
            'name' => 'header',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'the (optional) title to display at the top of the login page'
            ),
        array(
            'name' => 'introduction',
            'type' => 'text',
            'comment' => 'the (optional) introduction/explanation of this confab'
            ),
        array(
            'name' => 'footer',
            'type' => 'text',
            'comment' => 'the (optional) footer at the bottom of the login page'
            ),
        array(
            'name' => 'passcode',
            'type' => 'varchar',
            'length' => 80,
            'notnull' => TRUE,
            'comment' => 'the "secret" code visitors need to know to join the conversation'
            ),
        array(
            'name' => 'max_participants',
            'type' => 'int',
            'default' => 26,
            'notnull' => TRUE,
            'comment' => 'maximum # of participants in the conversation (default 26)'
            ),
        array(
            'name' => 'max_tries',
            'type' => 'int',
            'default' => 10,
            'notnull' => TRUE,
            'comment' => 'maximum # of tries allowed to join the conversation (default 10)'
            ),
        array(
            'name' => 'refresh',
            'type' => 'int',
            'default' => 6,
            'notnull' => TRUE,
            'comment' => 'page refresh interval (default 6 seconds)'
            ),
        array(
            'name' => 'timeout_idle',
            'type' => 'int',
            'default' => 300,
            'notnull' => TRUE,
            'comment' => 'time after which participants become idle (default 300 seconds)'
            ),
        array(
            'name' => 'timeout_conversation',
            'type' => 'int',
            'default' => 3600,
            'notnull' => TRUE,
            'comment' => 'time after which conversations are considered completed (default 3600 seconds)'
            ),
        array(
            'name' => 'status',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'status of the confab: 0=closed, 1=registerd users only, 2=opened'
            ),
        array(
            'name' => 'personal',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'person-to-person messages allowed: 0=no, 1=registered users, 2=yes'
            ),
        array(
            'name' => 'last_conversation_id',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'counter to distinguish conversations within this confab'
            ),
        array(
            'name' => 'last_participant_id',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'counter to distinguish participants within this conversation'
            ),
        array(
            'name' => 'ctime',
            'type' => 'datetime',
            'comment' => 'contains the time the confab was created'
            ),
        array(
            'name' => 'cuser_id',
            'type' => 'int',
            'comment' => 'identifies the user that created this confab'
            ),
        array(
            'name' => 'mtime',
            'type' => 'datetime',
            'comment' => 'contains the time the configuration was last updated'
            ),
        array(
            'name' => 'muser_id',
            'type' => 'int',
            'comment' => 'identifies the user that last modified the configuration'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('confab_id')
            ),
        array(
            'name' => 'node',
            'type' => 'foreign',
            'fields' => array('confab_id'),
            'reftable' => 'nodes',
            'reffields' => array('node_id')
            ),
        array(
            'name' => 'cuser',
            'type' => 'foreign',
            'fields' => array('cuser_id'),
            'reftable' => 'users',
            'reffields' => array('user_id')
            ),
        array(
            'name' => 'muser',
            'type' => 'foreign',
            'fields' => array('muser_id'),
            'reftable' => 'users',
            'reffields' => array('user_id')
            )
        )
    );
$tabledefs['confab_messages'] = array(
    'name' => 'confab_messages',
    'comment' => 'stores all visitor contributions',
    'fields' => array(
        array(
            'name' => 'confab_message_id',
            'type' => 'serial',
            'comment' => 'unique identification of a confab message'
            ),
        array(
            'name' => 'confab_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the confab to which this message belongs'
            ),
        array(
            'name' => 'conversation_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the conversation within this confab to which this message belongs'
            ),
        array(
            'name' => 'message_system',
            'type' => 'bool',
            'notnull' => TRUE,
            'default' => FALSE,
            'comment' => 'TRUE is a message from system/moderator, FALSE is an ordinary message'
            ),
        array(
            'name' => 'message_personal',
            'type' => 'bool',
            'notnull' => TRUE,
            'default' => FALSE,
            'comment' => 'TRUE is a private for message_to, FALSE is a public message to All'
            ),
        array(
            'name' => 'message_datim',
            'type' => 'datetime',
            'comment' => 'contains the server date/time the message was submitted'
            ),
        array(
            'name' => 'message_from_id',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => 'the participant-id of the sender or 0 if system (basis for attr)'
            ),
        array(
            'name' => 'message_from',
            'type' => 'varchar',
            'notnull' => FALSE,
            'default' => NULL,
            'length' => 72,
            'comment' => 'participant nickname/handle, NULL for messages from system/moderator'
            ),
        array(
            'name' => 'message_to',
            'type' => 'varchar',
            'notnull' => FALSE,
            'default' => NULL,
            'length' => 72,
            'comment' => 'participant nickname/handle, NULL for messages to all participants'
            ),
        array(
            'name' => 'message',
            'type' => 'text',
            'comment' => 'the actual content of the message'
            ),
        array(
            'name' => 'remote_addr',
            'type' => 'varchar',
            'length' => 150,
            'notnull' => TRUE,
            'comment' => 'IP-address of the visitor'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('confab_message_id')
            ),
        array(
            'name' => 'confab',
            'type' => 'foreign',
            'fields' => array('confab_id'),
            'reftable' => 'confabs',
            'reffields' => array('confab_id')
            ),
        array(
            'name' => 'confab_index',
            'type' => 'index',
            'fields' => array('confab_id','conversation_id','confab_message_id'),
            'comment' => 'hint for quick selections based on a conversation within a confab'
            )
        )
    );
$tabledefs['confab_participants'] = array(
    'name' => 'confab_participants',
    'comment' => 'transient housekeeping information for participant during the conversation',
    'fields' => array(
        array(
            'name' => 'confab_participant_id',
            'type' => 'serial',
            'comment' => 'unique identification of a participant'
            ),
        array(
            'name' => 'confab_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the confab in which this participant takes part'
            ),
        array(
            'name' => 'conversation_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the conversation within the confab'
            ),
        array(
            'name' => 'participant_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the participant within this conversation'
            ),
        array(
            'name' => 'interlocutor_id',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => 'personal interlocutress of this participant, 0 = all'
            ),
        array(
            'name' => 'confab_participant_key',
            'type' => 'varchar',
            'length' => 172,
            'comment' => 'unique identifier (\'token\') within confab stored in visitor\'s cookie'
            ),
        array(
            'name' => 'username',
            'type' => 'varchar',
            'length' => 64,
            'comment' => 'participant username, unique combined with username_ext'
            ),
        array(
            'name' => 'username_ext',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => '0 for anonymous users, >0 for registered users'
            ),
        array(
            'name' => 'nickname',
            'type' => 'varchar',
            'length' => 72,
            'comment' => 'participant nickname/handle, unique within the conversation'
            ),

        array(
            'name' => 'full_name',
            'type' => 'varchar',
            'length' => 255,
            'comment' => 'full name of participant (optional)'
            ),
        array(
            'name' => 'last_message_id',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => 'the last message served to the participant'
            ),
        array(
            'name' => 'last_seen',
            'type' => 'datetime',
            'comment' => 'contains date/time when participant was last seen'
            ),
        array(
            'name' => 'last_posted',
            'type' => 'datetime',
            'comment' => 'contains date/time of participant latest contribution'
            ),
        array(
            'name' => 'interface_id',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => 'user-selected confab client interface'
            ),
        array(
            'name' => 'status',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'participant status: 0=none, 1=joined, 2=leaving'
            ),
        array(
            'name' => 'show_list',
            'type' => 'bool',
            'notnull' => TRUE,
            'default' => FALSE,
            'comment' => 'TRUE means there are changes in the list of participants'
            ),
        array(
            'name' => 'is_active',
            'type' => 'bool',
            'notnull' => TRUE,
            'default' => FALSE,
            'comment' => 'TRUE means user has posted a message recently, FALSE means idle'
            )

        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('confab_participant_id')
            ),
        array(
            'name' => 'confab',
            'type' => 'foreign',
            'fields' => array('confab_id'),
            'reftable' => 'confabs',
            'reffields' => array('confab_id')
            ),
        array(
            'name' => 'confab_participant_index',
            'type' => 'index',
            'fields' => array('confab_participant_key','confab_id'),
            'unique' => TRUE
            ),
        array(
            'name' => 'confab_conversation_username_index',
            'type' => 'index',
            'fields' => array('confab_id','conversation_id','username','username_ext'),
            'unique' => TRUE
            )
        )
    );

?>