<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/htmlpage/htmlpage_view.php - interface to the view-part of the htmlpage module
 *
 * This file defines the interface with the htmlpage-module for viewing content.
 * The interface consists of this function:
 *
 * <code>
 * htmlpage_view(&$output,$area_id,$node_id,$module)
 * </code>
 *
 * This function is called from /index.php when the node to display is connected
 * to this module.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_htmlpage
 * @version $Id: htmlpage_view.php,v 1.7 2016/03/23 09:28:22 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }


/** display the content of the htmlpage linked to node $node_id
 *
 * @param object &$theme collects the (html) output
 * @param int $area_id identifies the area where $node_id lives
 * @param int $node_id the node to which this module is connected
 * @param array $module the module record straight from the database
 * @return bool TRUE on success + output via $theme, FALSE otherwise
 */
function htmlpage_view(&$theme,$area_id,$node_id,$module) {
    $table = 'htmlpages';
    $fields = array('page_data');
    $where = array('node_id' => intval($node_id));
    $order = array('version DESC');
    if (($record = db_select_single_record($table,$fields,$where,$order)) === FALSE) {
        $theme->add_message(t('error_retrieving_data','admin'));
        logger(sprintf("%s(): cannot retrieve node '%d': %s",__FUNCTION__,$node_id,db_errormessage()));
        $retval = FALSE;
    } else {
        $theme->add_content($record['page_data']);
        $retval = TRUE;
    }
    return $retval;
} // htmlpage_view()

?>