<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/mypage/mypage_admin.php - management interface for mypage-module
 *
 * This file defines the administrative interface to this module.
 * The interface consists of the following four functions.
 *
 * <code>
 * mypage_disconnect(&$output,$area_id,$node_id,$module)
 * mypage_connect(&$output,$area_id,$node_id,$module)
 * mypage_show_edit(&$output,$area_id,$node_id,$module,$viewonly,$edit_again,$href,$option)
 * mypage_save(&$output,$area_id,$node_id,$module,$viewonly,&$edit_again,$option)
 * </code>
 *
 * These functions are called from pagemanagerlib.php whenever necessary.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_mypage
 * @version $Id: mypage_admin.php,v 1.6 2016/03/22 13:08:08 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }


/** disconnect this module from a node
 *
 * this breaks the link between the node $node_id in area $area_id and this module.
 * Since there is nothing to configure this is a NOP.
 * 
 * @param object &$output collects the html output (if any)
 * @param int $area_id the area in which $node_id resides
 * @param int $node_id the node from which we need to disconnect
 * @param array $module the module record straight from the database
 * @return bool TRUE on success, FALSE otherwise
 */
function mypage_disconnect(&$output,$area_id,$node_id,$module) {
    return TRUE;
} // mypage_disconnect()


/** connect this module to a node
 *
 * this makes the link between the node $node_id in area $area_id and this module.
 * There is nothing to configure so this routine is a NOP.
 * 
 * @param object &$output collects the html output (if any)
 * @param int $area_id the area in which $node_id resides
 * @param int $node_id the node to which we need to connect
 * @param array $module the module record straight from the database
 * @return bool TRUE on success, FALSE otherwise
 */
function mypage_connect(&$output,$area_id,$node_id,$module) {
    return TRUE;
} // mypage_connect()


/** present the user with a dialog to modify the mypage that is connected to node $node_id
 *
 * display a short message indicating that this module has no configuration options.
 * The user can only use the Cancel button to return to Page Manager.
 *
 * @param object &$output collects the html output (if any)
 * @param int $area_id the area in which $node_id resides
 * @param int $node_id the node to which this module is connected
 * @param array $module the module record straight from the database
 * @param bool $viewonly if TRUE, editing is not allowed (but simply showing the content is allowed)
 * @param bool $edit_again if TRUE start with data from $_POST, else use data from database
 * @param string $href the action property of the HTML-form, the place where data will be POST'ed
 * @param string $option indicates which submenu-option was selected (NULL for none)
 * @return bool TRUE on success + output stored via $output, FALSE otherwise
 */
function mypage_show_edit(&$output,$area_id,$node_id,$module,$viewonly,$edit_again,$href,$option) {
    $dialogdef = array('button_cancel' => dialog_buttondef(BUTTON_CANCEL));
    $output->add_content('<h2>'.t('mypage_content_header','m_mypage').'</h2>');
    $output->add_content(t('mypage_content_explanation','m_mypage'));
    $output->add_content(dialog_quickform($href,$dialogdef));
    return TRUE;
} // mypage_show_edit()


/** save the modified content data of this module linked to node $node_id
 *
 * basically also a NOP:
 * If the user has cancelled the operation, the flag $edit_again is set to FALSE
 * and the return value is also FALSE.
 *
 * If the modified data is stored successfully, the return value is TRUE and
 * the value of $edit_again is based on the button used to save ([Save] or [Done]).
 *
 * Here is a summary of return values.
 *
 * <pre>
 * $retval | $edit_again | Action
 * ------- | ----------- | ------
 * FALSE   | FALSE       | [Cancel] Stop editing, unlock & return to tree view
 * FALSE   | TRUE        | [Save] or [Done] failed: redo the dialog and let user correct mistakes
 * TRUE    | FALSE       | [Done] pressed, everything saved succesfully, we're done
 * TRUE    | TRUE        | [Save] pressed, everything saved succesfully but continue editing
 * </pre>
 *
 * The parameter $option is not used in this module.
 *
 * @param object &$output collects the html output (if any)
 * @param int $area_id the area in which $node_id resides
 * @param int $node_id the node to which the content is connected
 * @param array $module the module record straight from the database
 * @param bool $viewonly if TRUE, editing and hence saving is not allowed
 * @param bool &$edit_again set to TRUE if we need to edit the content again, FALSE otherwise
 * @param string $option indicates which submenu-option was selected (NULL for none)
 * @return bool TRUE on success + output stored via $output, FALSE otherwise
 */
function mypage_save(&$output,$area_id,$node_id,$module,$viewonly,&$edit_again) {
    // 1 -- bail out if cancelled or viewonly
    if ((isset($_POST['button_cancel'])) || ($viewonly)) {
        $edit_again = FALSE;
        return FALSE;
    }
    return FALSE;
} // mypage_save()

?>