<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/newsletter/install/newsletter_tabledefs.php - data definition for module
 *
 * The following tables are defined:
 *
 * newsletters             - 1 record per newsletter, tied to a node; contains main configuration
 * newsletter_articles     - 0, 1 or more records containing unpublished articles
 * newsletter_issues       - 0, 1 or more records containing published newletter issues
 * newsletter_subscribers  - 0, 1 or more records with subscriber email addresses
 * newsletter_queue        - 0, 1 or more records with newsletters that need to be sent
 *
 * The tables are linked together via the newsletter_id which value is identical
 * to the node_id of the page to which this instance of the module is linked.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_newsletter
 * @version $Id: newsletter_tabledefs.php,v 1.6 2016/03/23 09:28:24 peter Exp $
 */
$tabledefs['newsletters'] = array(
    'name' => 'newsletters',
    'comment' => 'main table for newsletter module stores the configuration data',
    'fields' => array(
        /* basic module configuration */
        array(
            'name' => 'newsletter_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the node this newsletter is connected to'
            ),
        array(
            'name' => 'header',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'the (optional) title to display at the top of the page'
            ),
        array(
            'name' => 'introduction',
            'type' => 'text',
            'comment' => 'the (optional) introduction at the top of the page'
            ),
        array(
            'name' => 'footer',
            'type' => 'text',
            'comment' => 'the (optional) footer at the bottom of the page'
            ),
        array(
            'name' => 'email_administrators',
            'type' => 'text',
            'comment' => 'comma or newline-delimited list of newsletter administrator email addresses'
            ),
        /* newsletter configuration */
        array(
            'name' => 'title',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'title of the newsletter, eg. \'The Weekly Note\''
            ),
        array(
            'name' => 'subtitle',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'subtitle of the newsletter, eg. \'What\'s new at the Exemplum Primary School?\''
            ),
        array(
            'name' => 'newsletter_address',
            'type' => 'varchar',
            'length' => 255,
            'notnull' => TRUE,
            'comment' => 'the reply-to address used in every outgoing mail'
            ),
        array(
            'name' => 'newsletter_channel',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => '1=full-mail-only, 2=alert-only+site, 3=full-mail+site'
            ),
        array(
            'name' => 'newsletter_style',
            'type' => 'text',
            'notnull' => TRUE,
            'comment' => 'the (optional) additional bazaar style style (BSSS) of this newsletter'
            ),
        array(
            'name' => 'newsletter_subject',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => '(template) the email subject'
            ),
        array(
            'name' => 'newsletter_title',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => '(template) the HTML title'
            ),
        array(
            'name' => 'newsletter_header',
            'type' => 'text',
            'comment' => '(template) the (optional) header of this newsletter'
            ),
        array(
            'name' => 'newsletter_announce',
            'type' => 'text',
            'comment' => '(template) the (optional) announce message of this newsletter'
            ),
        array(
            'name' => 'newsletter_colofon',
            'type' => 'text',
            'comment' => '(template) the (optional) colofon of this newsletter'
            ),
        array(
            'name' => 'newsletter_footer',
            'type' => 'text',
            'comment' => '(template) the (optional) footer of this newsletter'
            ),
        /* user contributions */
        array(
            'name' => 'contribution_policy',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'contributions allowed: 0=no, 1=registered users, 2=yes'
            ),
        array(
            'name' => 'contribution_introduction',
            'type' => 'text',
            'comment' => 'instructions for contributing to the newsletter'
            ),
        /* subscriptions configuration */
        array(
            'name' => 'subscription_policy',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => '0=no subscriptions, 1=one-pass, 2=two step approval'
            ),
        array(
            'name' => 'subscription_introduction',
            'type' => 'text',
            'comment' => 'newsletter subscription instructions'
            ),
        /* runtime information */
        array(
            'name' => 'ctime',
            'type' => 'datetime',
            'comment' => 'contains the time the newsletter was created'
            ),
        array(
            'name' => 'cuser_id',
            'type' => 'int',
            'comment' => 'identifies the user that created this newsletter'
            ),
        array(
            'name' => 'mtime',
            'type' => 'datetime',
            'comment' => 'contains the time the configuration was last updated'
            ),
        array(
            'name' => 'muser_id',
            'type' => 'int',
            'comment' => 'identifies the user that last modified the configuration'
            ),
        array(
            'name' => 'next_volume',
            'type' => 'int',
            'default' => 0,
            'comment' => 'volume number for next newsletter'
            ),
        array(
            'name' => 'next_number',
            'type' => 'int',
            'default' => 0,
            'comment' => 'issue number for next newsletter in this volume'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('newsletter_id')
            ),
        array(
            'name' => 'node',
            'type' => 'foreign',
            'fields' => array('newsletter_id'),
            'reftable' => 'nodes',
            'reffields' => array('node_id')
            )
        )
    );
$tabledefs['newsletter_articles'] = array(
    'name' => 'newsletter_articles',
    'comment' => 'stores all articles until publication in a newsletter issue',
    'fields' => array(
        array(
            'name' => 'article_id',
            'type' => 'serial',
            'comment' => 'unique identification of a newsletter article'
            ),
        array(
            'name' => 'newsletter_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'newsletter to which this article belongs'
            ),
        array(
            'name' => 'title',
            'type' => 'varchar',
            'length' => 255,
            'comment' => 'title of the article'
            ),
        array(
            'name' => 'content',
            'type' => 'text',
            'comment' => 'actual content of the article'
            ),
        array(
            'name' => 'remarks',
            'type' => 'text',
            'comment' => 'author\'s remarks about the article (not to be published)'
            ),
        array(
            'name' => 'author_id',
            'type' => 'int',
            'notnull' => FALSE,
            'default' => 'NULL',
            'comment' => 'identifies the registered user that contributed, NULL=anonymous coward'
            ),
        array(
            'name' => 'author_full_name',
            'type' => 'varchar',
            'length' => 255,
            'comment' => 'registered user\'s full name OR a free-format (nick)name'
            ),
        array(
            'name' => 'email',
            'type' => 'varchar',
            'length' => 255,
            'comment' => 'registered user\'s e-mail address OR a user-defined e-mail address'
            ),
        array(
            'name' => 'datim',
            'type' => 'datetime',
            'comment' => 'contains the date/time the article was submitted'
            ),
        array(
            'name' => 'remote_addr',
            'type' => 'varchar',
            'length' => 150,
            'notnull' => TRUE,
            'comment' => 'IP-address of the author'
            ),
        array(
            'name' => 'selected',
            'type' => 'bool',
            'notnull' => TRUE,
            'default' => FALSE,
            'comment' => 'TRUE=select this article for next issue, FALSE=suppress'
            ),
        array(
            'name' => 'sort_order',
            'type' => 'int',
            'notnull' => TRUE,
            'default' => 0,
            'comment' => 'determines order in selected articles'
            ),
        array(
            'name' => 'mtime',
            'type' => 'datetime',
            'comment' => 'contains the time the article was last modified'
            ),
        array(
            'name' => 'muser_id',
            'type' => 'int',
            'comment' => 'identifies the user that last modified this article'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('article_id')
            ),
        array(
            'name' => 'newsletter',
            'type' => 'foreign',
            'fields' => array('newsletter_id'),
            'reftable' => 'newsletters',
            'reffields' => array('newsletter_id')
            ),
        array(
            'name' => 'article_index',
            'type' => 'index',
            'fields' => array('newsletter_id','article_id'),
            'comment' => 'hint for quick article selections within newsletter'
            )
        )
    );
$tabledefs['newsletter_issues'] = array(
    'name' => 'newsletter_issues',
    'comment' => 'stores all complete, published newsletters',
    'fields' => array(
        array(
            'name' => 'issue_id',
            'type' => 'serial',
            'comment' => 'unique identification of a newsletter issue'
            ),
        array(
            'name' => 'newsletter_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the newsletter to which this issue belongs'
            ),
        array(
            'name' => 'volume',
            'type' => 'int',
            'default' => 0,
            'comment' => 'volume number for this issue'
            ),
        array(
            'name' => 'number',
            'type' => 'int',
            'default' => 0,
            'comment' => 'issue number within this volume'
            ),
        array(
            'name' => 'datim',
            'type' => 'datetime',
            'comment' => 'contains issue publication date/time'
            ),
        array(
            'name' => 'toc',
            'type' => 'text',
            'comment' => 'table of contents (newline-delimited) of this newsletter issue'
            ),
        array(
            'name' => 'html_title',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'HTML title for this newsletter'
            ),
        array(
            'name' => 'html_style', 
            'type' => 'text', 
            'notnull' => TRUE,
            'comment' => 'additional style information for this newsletter issue'
            ),
        array(
            'name' => 'html_body',
            'type' => 'longtext',
            'comment' => 'the actual content of this newsletter issue (HTML)'
            ),
        array(
            'name' => 'mail_subject',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'the email subject for this newsletter'
            ),
        array(
            'name' => 'mail_body',
            'type' => 'longtext',
            'comment' => 'the actual content of this newsletter issue (E-Mail)'
            ),
        array(
            'name' => 'mail_headers',
            'type' => 'text',
            'notnull' => TRUE,
            'comment' => 'newline-delimited list of headers to be added'
            ),
        array(
            'name' => 'status',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => '0=unpublished, 1=full-mail-only, 2=alert-only+site, 3=full-mail+site'
            ),
        array(
            'name' => 'ptime',
            'type' => 'datetime',
            'comment' => 'contains the time the issue was approved for publication'
            ),
        array(
            'name' => 'puser_id',
            'type' => 'int',
            'comment' => 'identifies the user that published this issue'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('issue_id')
            ),
        array(
            'name' => 'newsletter',
            'type' => 'foreign',
            'fields' => array('newsletter_id'),
            'reftable' => 'newsletters',
            'reffields' => array('newsletter_id')
            ),
        array(
            'name' => 'issue_index',
            'type' => 'index',
            'fields' => array('newsletter_id','volume','number'),
            'comment' => 'hint for quick issue selections'
            )
        )
    );
$tabledefs['newsletter_subscribers'] = array(
    'name' => 'newsletter_subscribers',
    'comment' => 'stores subscriber information',
    'fields' => array(
        array(
            'name' => 'subscriber_id',
            'type' => 'serial',
            'comment' => 'unique identification of a subscriber'
            ),
        array(
            'name' => 'newsletter_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the newsletter the subscriber is associated with'
            ),
        array(
            'name' => 'email',
            'type' => 'varchar',
            'length' => 255,
            'notnull' => TRUE,
            'comment' => 'subscriber\'s e-mail address'
            ),
        array(
            'name' => 'full_name',
            'type' => 'varchar',
            'length' => 255,
            'comment' => 'subscriber\'s full name'
            ),
        array(
            'name' => 'remarks',
            'type' => 'text',
            'comment' => 'the (optional) additional information about subscriber'
            ),
        array(
            'name' => 'status',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => '0=new, 1=confirmed, 2=approved, 3=blacklisted'
            ),
        array(
            'name' => 'oncecode',
            'type' => 'varchar',
            'length' => '255',
            'notnull' => FALSE,
            'default' => 'NULL',
            'comment' => 'quasi random string used for email confirmations'
            ),
        array(
            'name' => 'oncecode_expiry',
            'type' => 'datetime',
            'notnull' => FALSE,
            'default' => 'NULL',
            'comment' => 'contains the time when the one-time code expires'
            ),
        array(
            'name' => 'oncecode_errors',
            'type' => 'int',
            'default' => 0,
            'comment' => 'keeps track of # of attempts to enter with the wrong code'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('subscriber_id')
            ),
        array(
            'name' => 'newsletter',
            'type' => 'foreign',
            'fields' => array('newsletter_id'),
            'reftable' => 'newsletters',
            'reffields' => array('newsletter_id')
            )
        )
    );
$tabledefs['newsletter_queue'] = array(
    'name' => 'newsletter_queue',
    'comment' => 'stores pointers to all outgoing newsletter issues',
    'fields' => array(
        array(
            'name' => 'queue_id',
            'type' => 'serial',
            'comment' => 'unique identification of a queued newsletter issue'
            ),
        array(
            'name' => 'issue_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'link to the newsletter issue to send out'
            ),
        array(
            'name' => 'subscriber_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the subscriber to send the newsletter issue to'
            ),
        array(
            'name' => 'errors',
            'type' => 'int',
            'default' => 0,
            'comment' => 'counter for failed delivery attempts'
            ),
        array(
            'name' => 'retry',
            'type' => 'datetime',
            'comment' => 'retry delivery after this date/time'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('queue_id')
            ),
        array(
            'name' => 'issue',
            'type' => 'foreign',
            'fields' => array('issue_id'),
            'reftable' => 'newsletter_issues',
            'reffields' => array('issue_id')
            ),
        array(
            'name' => 'subscriber',
            'type' => 'foreign',
            'fields' => array('subscriber_id'),
            'reftable' => 'newsletter_subscribers',
            'reffields' => array('subscriber_id')
            )
        )
    );

?>