<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/newsletter/newsletter_cron.php - interface to the cron-part of the newsletter module
 *
 * This file defines the interface with the newsletter-module for cron.
 * The interface consists of this function:
 *
 * <code>
 * newsletter_cron()
 * </code>
 *
 * This function is called whenever cron determines that it is time to perform this function.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_newsletter
 * @version $Id: newsletter_cron.php,v 1.4 2016/03/23 09:28:24 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }

require_once(dirname(__FILE__).'/newsletter_common.php'); // common code + constants

/** routine that is called periodically by cron
 *
 * this routine performs household chores every now and then (interval: 15 minutes;
 * see {@link newsletter_manifest.php}). An important chore is tending to any
 * queued newsletters. Another one is getting rid of obsolete and unfinished
 * new entries in newsletter_subscribers and existing entries with expired
 * oncecodes.
 *
 * @return bool TRUE on success, FALSE otherwise
 */
function newsletter_cron() {
    $retval = TRUE;
    $now = strftime('%Y-%m-%d %T');
    $table = 'newsletter_subscribers';

    // 1 -- remove new subscribers with expired oncecode
    $where = sprintf('(status = %d) AND (oncecode_expiry < %s)',
                     NEWSLETTER_SUBSCRIPTION_NEW, db_escape_and_quote($now));
    if (($n = db_delete($table,$where)) === FALSE) {
        logger(sprintf('%s(): cannot delete expired new subscribers: %s',__FUNCTION__,db_errormessage()));
        $retval = FALSE;
    } else {
        logger(sprintf('%s(): success deleting %d new subscribers (expired)',__FUNCTION__,$n),WLOG_DEBUG);
    }

    // 2 -- reset expired oncecodes
    $fields = array('oncecode' => NULL,'oncecode_expiry' => NULL,'oncecode_errors' => 0);
    $where = sprintf('(oncecode_expiry < %s)', db_escape_and_quote($now));
    if (($n = db_update($table,$fields,$where)) === FALSE) {
        logger(sprintf('%s(): cannot reset expired oncecodes: %s',__FUNCTION__,db_errormessage()));
        $retval = FALSE;
    } else {
        logger(sprintf('%s(): success resetting %d expired oncecodes',__FUNCTION__,$n),WLOG_DEBUG);
    }

    // 3 -- send out pending newsletters
    if (newsletter_queue_run() === FALSE) {
        $retval = FALSE;
    }

    // 4 -- done
    return $retval;
} // newsletter_cron()

?>