<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/newsletter/newsletter_install.php - installer of the newsletter module
 *
 * This file contains the newsletter module installer.
 * The interface consists of these functions:
 *
 * <code>
 * newsletter_install(&$messages,$module_id)
 * newsletter_upgrade(&$messages,$module_id)
 * newsletter_uninstall(&$messages,$module_id)
 * newsletter_demodata(&$messages,$module_id,&$config,$manifest)
 * </code>
 *
 * These functions can be called from the main installer and/or admin.php.
 *
 * Note
 * You cannot be sure about the environment from which these routines are called.
 * If the caller is the Install Wizard, you do not have all subroutines available.
 * However, it IS possible to manipulate the database via the db_*() routines and/or
 * the global $DB object. Therefore you have to keep the install routine extremly
 * simple. You also have no option to interact with the user; the install has to
 * be a silent install; you can only indicate success (TRUE) or failure (FALSE)
 * and maybe an error message in $messages[] but that's it. Good luck.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_newsletter
 * @version $Id: newsletter_install.php,v 1.8 2016/06/16 13:48:31 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }


/** install the module
 *
 * this routine installs the module. For this module
 * there is nothing to install, so we simply return success.
 *
 * Note that the record for this module is already created in the
 * modules table; the pkey is $module_id.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function newsletter_install(&$messages,$module_id) {
    return TRUE;
} // newsletter_install()


/** upgrade the module
 *
 * this routine performs an upgrade to the installed module.
 * Note that the newsletter module does not need any upgrade at
 * all because there never was an earlier version.
 *
 * For now this routine is a nop.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function newsletter_upgrade(&$messages,$module_id) {
    return TRUE;
} // newsletter_upgrade()


/** uninstall the module
 *
 * this is a hook for future extensions of Website@School.
 * For now we simply return success.
 * 
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 */
function newsletter_uninstall(&$messages,$module_id) {
    return TRUE;
} // newsletter_uninstall()


/** add demonstration data to the system
 *
 * this routine adds demodata for this module to the database.
 *
 * Note that a routine by this name should exist (even if it does nothing).
 *
 * Note
 * If the module is installed via the Install Wizard, this routine is
 * called. However, if a module is installed as an additional module
 * after installation, the {$module}_demodata() routine is never called.
 * This is because the only time you know that demodata is installed is
 * when the Install Wizard runs. If we're called from admin.php, the
 * webmaster may have already deleted existing (core) demodata so you
 * never can be sure what to expect. To put it another way: it is hard
 * to re-construct $config when we're NOT the Instal Wizard.
 *
 * The array $config contains the following information.
 *
 * <code>
 * $config['language_key']   => install language code (eg. 'en')
 * $config['dir']            => path to CMS Root Directory (eg. /home/httpd/htdocs)
 * $config['www']            => URL of CMS Root Directory (eg. http://exemplum.eu)
 * $config['progdir']        => path to program directory (eg. /home/httpd/htdocs/program)
 * $config['progwww']        => URL of program directory (eg. http://exemplum.eu/program)
 * $config['datadir']        => path to data directory (eg. /home/httpd/wasdata/a1b2c3d4e5f6)
 * $config['user_username']  => userid of webmaster (eg. wblader)
 * $config['user_full_name'] => full name of webmaster (eg. Wilhelmina Bladergroen)
 * $config['user_email']     => email of webmaster (eg. w.bladergroen@exemplum.eu)
 * $config['user_id']        => numerical user_id (usually 1)
 * $config['demo_salt']      => password salt for all demodata accounts
 * $config['demo_password']  => password for all demodata accounts
 * $config['demo_areas']     => array with demo area data
 * $config['demo_groups']    => array with demo group data
 * $config['demo_users']     => array with demo user data
 * $config['demo_nodes']     => array with demo node data
 * $config['demo_string']    => array with demo strings from /program/install/languages/LL/demodata.php
 * $config['demo_replace']   => array with search/replace pairs to 'jazz up' the demo strings
 * </code>
 *
 * With this information, we can add a demonstration configuration for the public area,
 * which shows off the possibilities. Note that we add our own additions to the array
 * $config so other modules and themes can determine the correct status quo w.r.t. the
 * demodata nodes etc.
 *
 * @param array &$messages collects the (error) messages
 * @param int $module_id the key for this module in the modules table
 * @param array &$config pertinent data for the new website + demodata foundation
 * @param array $manifest a copy from the manifest for this module
 * @return bool TRUE on success + output via $messages, FALSE otherwise
 * @todo There are various stubs that need to be fixed.
 */
function newsletter_demodata(&$messages,$module_id,&$config,$manifest) {
    global $DB;
    $retval = TRUE; // assume success

    $node_style = '.newsletter_header h1 { display: none; }
.newsletter_toc_header { background: #FFAAAA; }
.newsletter_article_header { background: #AAAAFF; }
.newsletter_colofon_header { background: #FFAAAA; }
#newsletter_view_navigation { background-color: #AAFFAA; }
#newsletter_view_navigation a { font-weight: bold; margin: 15px; }
'; // BSSS for node
    $style = '.newsletter_toc_header { background: #AAAAFF; }
.newsletter_article_header { background: #AAFFAA; }
.newsletter_colofon_outer { background: #FFFFAA; }
.newsletter_article_author {
  font-weight: bold;
  background: #FFAAFF;
}
'; // BSSS for newsletter configuration and archive

    // 0 -- get hold of the module-specific translations in $string[]
    $string = array();
    $filename = dirname(__FILE__).'/languages/en/newsletter.php'; // Always use English first
    if (file_exists($filename)) {
        @include($filename);
    }
    if (($language_key = $config['language_key']) != 'en') {
        $filename = dirname(__FILE__).'/languages/'.$language_key.'/newsletter.php';
        if (file_exists($filename)) {
            @include($filename);
        }
    }
    if (empty($string)) {
        $messages[] = 'Internal error: no translations in '.$filename;
        return FALSE;
    }

    // 1A -- prepare a demo-node for our newsletter module as a child from 'showcase'
    $sort_order = 0;
    $parent_id = $config['demo_nodes']['showcase']['node_id'];
    foreach($config['demo_nodes'] as $node => $fields) {
        if (($fields['parent_id'] == $parent_id) && ($fields['node_id'] != $parent_id)) {
            $sort_order = max($sort_order,$fields['sort_order']);
        }
    }
    $sort_order += 10; // place the althing-section at the end of the parent section
    $nodes = array(
        'newsletter' => array(
            'parent_id'  => $parent_id,
            'is_page'    => TRUE,
            'is_default' => FALSE,
            'title'      => strtr($string['demo_node_title'],$config['demo_replace']),
            'link_text'  => strtr($string['demo_node_link_text'],$config['demo_replace']),
            'sort_order' => $sort_order,
            'module_id'  => $module_id,
            'style'      => $node_style)
        );

    // 1B -- actually add the necessary node
    $now = strftime('%Y-%m-%d %T');
    $user_id   = $config['user_id'];
    $area_id   = $config['demo_areas']['public']['area_id'];
    foreach($nodes as $node => $fields) {
        $fields['area_id']  = $area_id;
        $fields['ctime']    = $now;
        $fields['mtime']    = $now;
        $fields['atime']    = $now;
        $fields['owner_id'] = $user_id;
        if (($node_id = db_insert_into_and_get_id('nodes',$fields,'node_id')) === FALSE) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        }
        $node_id = intval($node_id);
        $fields['node_id'] = $node_id;
        $config['demo_nodes'][$node] = $fields;
    }

    // 1C -- setup permissions for Andrew, Catherine (Seniors) and Amelia Cackle (Faculty)
    $seniors_id = intval($config['demo_groups']['seniors']['group_id']);
    $faculty_id = intval($config['demo_groups']['faculty']['group_id']);
    $sql = sprintf('SELECT gc.capacity_code, gc.acl_id, a.permissions_jobs '.
                   'FROM %sgroups_capacities gc JOIN %sacls a '.
                   'USING (acl_id) '.
                   'WHERE (gc.group_id = %d) OR (gc.group_id = %d)',
                   $DB->prefix,$DB->prefix,$seniors_id,$faculty_id);
    if (($DBResult = $DB->query($sql)) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    } elseif (($records = $DBResult->fetch_all_assoc()) === FALSE) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $DBResult->close();
        $retval = FALSE;
    } else {
        $DBResult->close();
        foreach($records as $record) {
            $capacity_code = intval($record['capacity_code']);
            $acl_id = intval($record['acl_id']);
            $permissions_jobs = intval($record['permissions_jobs']);
            if ($capacity_code == 1) {
                $permissions_jobs |= 7;      // Seniors/Pupils: Startcenter+PageManager+FileManager
                $permissions_nodes = 1;      // PageManager: Content Master (for $node_id)
            } elseif ($capacity_code == 3) {
                $permissions_jobs |= 7;      // Faculty/Principal: Startcenter+PageManager+FileManager
                $permissions_nodes = 15;     // PageManager: Page Master (for $node_id)
            } else {
                $permissions_nodes = 0;
            }
            $where = array('acl_id' => $acl_id);
            $fields = array('permissions_jobs' => $permissions_jobs);
            if (db_update('acls',$fields,$where) === FALSE) {
                $messages[] = $config['demo_string']['error'].db_errormessage();
                $retval = FALSE;
            }
            $fields = array(
                'acl_id' => $acl_id,
                'node_id' => intval($node_id),
                'permissions_modules' => 0,
                'permissions_nodes' => $permissions_nodes);
            if (db_insert_into('acls_nodes',$fields) === FALSE) {
                $messages[] = $config['demo_string']['error'].db_errormessage();
                $retval = FALSE;
            }
        }
    }

    // 2A -- create a new newsletter configuration
    $andrew = $config['demo_users']['andrew']['full_name'];
    $catherine = $config['demo_users']['catherine']['full_name'];
    $amelia = $config['demo_users']['acackl']['full_name'];
    $params = array('{ANDREW}' => $andrew, '{CATHERINE}' => $catherine, '{AMELIA}' => $amelia);
    $table = 'newsletters';
    $newsletter_config = array(
        'newsletter_id' => $node_id,
        'header' => $string['demo_node_title'],
        'introduction' => '',
        'footer' => '',
        'email_administrators' => $config['user_email'],
        'title' => $string['demo_node_title'],
        'subtitle' => $string['demo_subtitle'],
        'newsletter_address' => $config['user_email'],
        'newsletter_channel' => 3, // NEWSLETTER_FULLMAIL_SITE
        'newsletter_style' => $style,
        'newsletter_subject' => $string['newsletter_subject_template'],
        'newsletter_title' => $string['newsletter_title_template'],
        'newsletter_header' => $string['newsletter_header_template'],
        'newsletter_announce' => $string['newsletter_announce_template'],
        'newsletter_colofon' => strtr($string['demo_colofon'],$params),
        'newsletter_footer' => '<hr>{TITLE} {UNSUBSCRIBE}', // KISS
        'contribution_policy' => 1, // NEWSLETTER_CONTRIBUTION_POLICY_USERS
        'contribution_introduction' => $string['demo_contribution'],
        'subscription_policy' => 2, // NEWSLETTER_SUBSCRIPTION_POLICY_TWO
        'subscription_introduction' => $string['demo_subscription'],
        'ctime' => $now,
        'cuser_id' => $user_id,
        'mtime' => $now,
        'muser_id' => $user_id,
        'next_volume' => ($volume = intval($config['demo_replace']['{YEAR}'])),
        'next_number' => 5
        );
    if (db_insert_into($table,$newsletter_config) !== 1) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }
    
    // 2B -- insert at least 1 subscriber
    $table = 'newsletter_subscribers';
    $fields = array(
        'newsletter_id' => $node_id,
        'email' => $config['user_email'],
        'full_name' => $config['user_full_name'],
        'remarks' => '',
        'status' => 2 //NEWSLETTER_SUBSCRIPTION_APPROVED
        );
    if (db_insert_into($table,$fields) !== 1) {
        $messages[] = $config['demo_string']['error'].db_errormessage();
        $retval = FALSE;
    }

    // 3 -- create 4 'old' monthly newsletters
    $newsletters = array(
        1 => array(
            1 => array('title' => $string['demo_from_the_principal'], 'author' => $amelia,
                       'body' => '<p>{LOREM} {IPSUM}</p><p>{DOLOR} {SIT}</p><p>{IPSUM} {SIT}</p>'),
            2 => array('title' => $string['demo_upcoming_events'], 'author' => $andrew,
                       'body' => '<p>{DOLOR} {SIT}</p><p>{LOREM} {IPSUM}</p><p>{IPSUM} {SIT}</p>'),
            3 => array('title' => $string['demo_art_classes_juniors'], 'author' => $andrew,
                       'body' => '<p>{SIT} {LOREM} {DOLOR}</p>'),
            4 => array('title' => $string['demo_did_you_know'], 'author' => $catherine,
                       'body' => '<p>{SIT}</p><ul><li>{DOLOR}<li>{IPSUM}</ul><p>{LOREM}</p>'),
            5 => array('title' => $string['newsletter_colofon'], 'author' => '',
                       'body' => $newsletter_config['newsletter_colofon'])),
        2 => array(
            1 => array('title' => $string['demo_from_the_principal'], 'author' => $amelia,
                       'body' => '<p>{IPSUM} {LOREM}</p><p>{SIT} {DOLOR}</p><p>{SIT} {IPSUM}</p>'),
            2 => array('title' => $string['demo_upcoming_events'], 'author' => $andrew,
                       'body' => '<p>{SIT} {DOLOR}</p><p>{IPSUM} {LOREM}</p><p>{SIT} {IPSUM}</p>'),
            3 => array('title' => $string['demo_new_teacher'], 'author' => $amelia,
                       'body' => '<p>{LOREM} {DOLOR} {SIT}</p>'),
            4 => array('title' => $string['demo_did_you_know'], 'author' => $catherine,
                       'body' => '<p>{DOLOR}</p><ul><li>{SIT}<li>{IPSUM}</ul><p>{LOREM}</p>'),
            5 => array('title' => $string['newsletter_colofon'], 'author' => '',
                       'body' => $newsletter_config['newsletter_colofon'])),
        3 => array(
            1 => array('title' => $string['demo_from_the_principal'],
                       'author' => $amelia,
                       'body' => '<p>{SIT} {IPSUM}</p><p>{LOREM} {IPSUM}</p><p>{SIT} {DOLOR}</p>'),
            2 => array('title' => $string['demo_plans_new_year'], 'author' => $amelia,
                       'body' => '<p>{LOREM} {DOLOR} {SIT}</p>'),
            3 => array('title' => $string['demo_upcoming_events'], 'author' => $andrew,
                       'body' => '<p>{SIT} {DOLOR}</p><p>{LOREM} {IPSUM}</p><p>{SIT} {IPSUM}</p>'),
            4 => array('title' => $string['demo_did_you_know'], 'author' => $catherine,
                       'body' => '<p>{SIT}</p><ul><li>{DOLOR}<li>{IPSUM}</ul><p>{LOREM}</p>'),
            5 => array('title' => $string['newsletter_colofon'], 'author' => '',
                       'body' => $newsletter_config['newsletter_colofon'])),
        4 => array(
            1 => array('title' => $string['demo_from_the_principal'], 'author' => $amelia,
                       'body' => '<p>{SIT} {IPSUM}</p><p>{LOREM} {IPSUM}</p><p>{SIT} {DOLOR}</p>'),
            2 => array('title' => $string['demo_excursion_title'], 'author' => $andrew,
                       'body' => '<p>'.$string['demo_excursion_body'].'</p><p>{LOREM} {DOLOR} {SIT}</p>'),
            3 => array('title' => $string['demo_upcoming_events'], 'author' => $andrew,
                       'body' => '<p>{SIT}</p><p>'.$string['demo_excursion'].'</p><p>{IPSUM}</p>'),
            4 => array('title' => $string['demo_did_you_know'], 'author' => $catherine,
                       'body' => '<p>{SIT}</p><ul><li>{DOLOR}<li>{IPSUM}</ul><p>{LOREM}</p>'),
            5 => array('title' => $string['newsletter_colofon'], 'author' => '',
                       'body' => $newsletter_config['newsletter_colofon']))
        );
    for ($number=1; $number<5; ++$number) {
        // prepare 4 newsletters, starting at the oldest (4 months ago).
        $m = 5 - $number;
        $date = $config['demo_replace']["{MONTHS_AGO_$m}"];
        $datim = $date.' 09:00:00';
        $params = array(
            '{TITLE}' => $string['demo_node_title'],
            '{SUBTITLE}' => $string['demo_subtitle'],
            '{VOLUME}' => $volume,
            '{NUMBER}' => $number,
            '{DATE}' => $date,
            '{UNSUBSCRIBE}' => sprintf('<a href="%s?node=%d&unsubscribe=1">%s</a>',
                                       $config['demo_replace']['{INDEX_URL}'],
                                       $node_id,$string['view_unsubscribe']));
        // construct text-only toc + header + ToC
        $toc = '';
        $body = "<a name=\"top\"></a>\n".
                "<div class=\"newsletter_header\">\n".
                strtr($newsletter_config['newsletter_header'],$params)."\n</div>\n".
                "<div class=\"newsletter_toc_outer\" id=\"newsletter_outer_0\">\n".
                "<h3 class=\"newsletter_toc_header\" id=\"newsletter_header_0\">".
                "<a name=\"toc\" href=\"#top\">".
                $string['newsletter_toc_title']."</a></h3>\n".
                "<ul class=\"newsletter_toc_inner\" id=\"newsletter_inner_0\">\n";
        foreach ($newsletters[$number] as $k => $v) {
            $toc .= $v['title']."\n";
            $body .= sprintf("  <li><a href=\"#h%d\">%s</a></li>\n",$k,$v['title']);
        }
        $body .= "</ul>\n</div>\n";
        $toc = trim($toc);
        
        // step through the articles now
        foreach ($newsletters[$number] as $k => $v) {
            $article = ($k == 5) ? 'colofon' : 'article';
            $body .= "<div class=\"newsletter_${article}_outer\" id=\"newsletter_outer_$k\">\n".
                     "<h3 class=\"newsletter_${article}_header\" id=\"newsletter_header_$k\">".
                     "<a name=\"h$k\" href=\"#toc\">".$v['title']."</a></h3>\n";
            $author = trim($v['author']);
            if (!empty($author)) {
                $body .= "<div class=\"newsletter_article_author\" id=\"newsletter_author_$k\">$author</div>\n";
            }
            $body .= "<div class=\"newsletter_${article}_inner\" id=\"newsletter_inner_$k\">\n".
                     strtr($v['body'],$config['demo_replace']).
                     "\n</div>\n</div>\n";
        }
        $body .= "<div class=\"newsletter_footer\">\n".
                 strtr($newsletter_config['newsletter_footer'],$params).
                 "\n</div>\n";
        // store this body in the issues table (creating a demo archive of 1 volume)
        $table = 'newsletter_issues';
        $fields = array(
            'newsletter_id' => $node_id,
            'volume' => $volume,
            'number' => $number,
            'datim' => $datim,
            'toc' => $toc,
            'html_title' => strtr($string['newsletter_title_template'],$params),
            'html_style' => $style,
            'html_body' => $body,
            'mail_subject' => strtr($string['newsletter_subject_template'],$params),
            'mail_body' => 'Demodata: no mail body in demo',
            'mail_headers' => 'X-Demodata: No mail headers in demo',
            'status' => 3, // NEWSLETTER_FULLMAIL_SITE
            'ptime' => $datim,
            'puser_id' => $config['demo_users']['acackl']['user_id']
            );
        if (db_insert_into($table,$fields) !== 1) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        }
    }

    // Done with the archive, insert a few pending articles for upcoming issues.
    $table = 'newsletter_articles';
    $articles = array(
        array(
            'title' => $string['demo_from_the_principal'],
            'content' => '<p>{LOREM} {IPSUM}</p><p>{DOLOR} {SIT}</p><p>{IPSUM} {SIT}</p>',
            'remarks' => '',
            'author_id' => intval($config['demo_users']['acackl']['user_id']),
            'author_full_name' => $amelia,
            'email' => $config['demo_users']['acackl']['email'],
            'datim' => strftime('%Y-%m-%d %T',time()-4096),
            'selected' => TRUE,
            'mtime' => strftime('%Y-%m-%d %T',time()-4096),
            'muser_id' => intval($config['demo_users']['acackl']['user_id']),
            ),
        array(
            'title' => $string['demo_upcoming_events'],
            'content' => '<p>{IPSUM} {LOREM}</p><p>{SIT} {DOLOR}</p><p>{IPSUM} {SIT}</p>',
            'remarks' => '',
            'author_id' => intval($config['demo_users']['andrew']['user_id']),
            'author_full_name' => $andrew,
            'email' => $config['demo_users']['andrew']['email'],
            'datim' => strftime('%Y-%m-%d %T',time()-3072),
            'selected' => TRUE,
            'mtime' => strftime('%Y-%m-%d %T',time()-3072),
            'muser_id' => intval($config['demo_users']['andrew']['user_id']),
            ),
        array(
            'title' => $string['demo_visit_botanical_garden'],
            'content' => '<p>{DOLOR} {LAST_WEEK}.</p><p>{SIT} {IPSUM}</p>',
            'remarks' => '',
            'author_id' => intval($config['demo_users']['catherine']['user_id']),
            'author_full_name' => $catherine,
            'email' => $config['demo_users']['catherine']['email'],
            'datim' => strftime('%Y-%m-%d %T',time()-2560),
            'selected' => TRUE,
            'mtime' => strftime('%Y-%m-%d %T',time()-2560),
            'muser_id' => intval($config['demo_users']['catherine']['user_id']),
            ),
        array(
            'title' => $string['demo_did_you_know'],
            'content' => '<p>{DOLOR}</p><ul><li>{SIT}<li>{IPSUM}</ul><p>{LOREM}</p>',
            'remarks' => '',
            'author_id' => intval($config['demo_users']['catherine']['user_id']),
            'author_full_name' => $catherine,
            'email' => $config['demo_users']['catherine']['email'],
            'datim' => strftime('%Y-%m-%d %T',time()-2048),
            'selected' => TRUE,
            'mtime' => strftime('%Y-%m-%d %T',time()-2048),
            'muser_id' => intval($config['demo_users']['catherine']['user_id']),
            ),
        array(
            'title' => $string['demo_chess_title'],
            'content' => $string['demo_chess_content'],
            'remarks' => $string['demo_chess_remarks'],
            'author_id' => NULL,
            'author_full_name' => $config['demo_users']['mmonte']['full_name'],
            'email' => $config['demo_users']['mmonte']['email'],
            'datim' => strftime('%Y-%m-%d %T',time()-1024),
            'selected' => FALSE,
            'mtime' => NULL,
            'muser_id' => NULL
            )
        );
    $sort_order = 0;
    foreach($articles as $fields) {
        $sort_order += 10;
        $fields['newsletter_id'] = $node_id;
        $fields['remote_addr'] = '127.0.0.1';
        $fields['sort_order'] = $sort_order;
        $fields['title'] = strtr($fields['title'],$config['demo_replace']);
        $fields['content'] = strtr($fields['content'],$config['demo_replace']);
        if (db_insert_into($table,$fields) !== 1) {
            $messages[] = $config['demo_string']['error'].db_errormessage();
            $retval = FALSE;
        }
    }
    // Finally, adjust the two redirects that were prepared to point to 
    // this newsletter from the 'News' section (see /program/install/demodata.php)

    $friendly = $config['friendly_url'];
    $redirects = array(
        $config['demo_nodes']['latestnewsletter']['node_id'] => array(
            'link_href' => sprintf(($friendly)?'%s/%d':'%s?node=%d',
                                   $config['demo_replace']['{INDEX_URL}'],$node_id)),
        $config['demo_nodes']['oldnewsletters']['node_id'] => array(
            'link_href' => sprintf(($friendly)?'%s/%d/volume/%d':'%s?node=%d&amp;volume=%d',
                                   $config['demo_replace']['{INDEX_URL}'],$node_id,$volume))
        );
    foreach($redirects as $node => $fields) {
        if (db_update('nodes', $fields, array('node_id' => $node)) === FALSE) {
            $messages[] = $tr['error'].' '.db_errormessage();
            $retval = FALSE;
        }
    }

    // all done

    return $retval;
} // newsletter_demodata()

?>