<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/modules/search/install/search_tabledefs.php - data definition for module
 *
 * One table is defined: search. It contains the configuration data for a search
 * (header, introduction, etc.)
 *
 * This module also uses a few temporary tables. These are created on the fly
 * and eventually lead to a table that contains the nodes that are to be
 * searched. The structure of that table looks like this:
 *
 * <code>
 * CREATE TABLE search_nodes (node_id int(11) NOT NULL DEFAULT '0' PRIMARY KEY (node_id))
 * </code>
 *
 * It is created in {@link search_construct_nodelist()} and used in all modules,
 * see {@link search_module()}.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wasmod_search
 * @version $Id: search_tabledefs.php,v 1.1 2016/04/01 18:31:50 peter Exp $
 */
$tabledefs['search'] = array(
    'name' => 'search',
    'comment' => 'main table for search module stores the configuration data',
    'fields' => array(
        array(
            'name' => 'node_id',
            'type' => 'int',
            'notnull' => TRUE,
            'comment' => 'the node this search is connected to'
            ),
        array(
            'name' => 'header',
            'type' => 'varchar',
            'length' => 240,
            'notnull' => TRUE,
            'comment' => 'the (optional) title to display at the top of the page'
            ),
        array(
            'name' => 'introduction',
            'type' => 'text',
            'comment' => 'the (optional) introduction/explanation above the page'
            ),
        array(
            'name' => 'scope',
            'type' => 'int',
            'default' => 0,
            'notnull' => TRUE,
            'comment' => 'search scope: 0=this area, 1=public areas, 2=all available areas'
            ),
        array(
            'name' => 'results',
            'type' => 'int',
            'default' => 20,
            'notnull' => TRUE,
            'comment' => 'results per page'
            ),
        array(
            'name' => 'ctime',
            'type' => 'datetime',
            'comment' => 'contains the time the search was created'
            ),
        array(
            'name' => 'cuser_id',
            'type' => 'int',
            'comment' => 'identifies the user that created this search'
            ),
        array(
            'name' => 'mtime',
            'type' => 'datetime',
            'comment' => 'contains the time the configuration was last updated'
            ),
        array(
            'name' => 'muser_id',
            'type' => 'int',
            'comment' => 'identifies the user that last modified the configuration'
            )
        ),
    'keys' => array(
        array(
            'type' => 'primary',
            'fields' => array('node_id')
            ),
        array(
            'name' => 'node',
            'type' => 'foreign',
            'fields' => array('node_id'),
            'reftable' => 'nodes',
            'reffields' => array('node_id')
            )
        )
    );

?>