<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** /program/themes/ruta/ruta.class.php - the class that implements the theme
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wastheme_ruta
 * @version $Id: ruta.class.php,v 1.1 2016/03/08 19:29:07 peter Exp $
 */
if (!defined('WASENTRY')) { die('no entry'); }

class ThemeRuta extends Theme {

    /** @var string the additional style that implements the rotating banner background */
    var $ruta_header_background = '';

    /** construct a ThemeRuta object
     *
     * @see Theme()
     * @param array $theme_record the record straight from the database
     * @param int $area_id the area of interest
     * @param int $node_id the node that will be displayed
     * @return void
     */
    function ThemeRuta($theme_record,$area_id,$node_id) {
        parent::Theme($theme_record,$area_id,$node_id);
        $this->quickbottom_separator = '|';
        $this->quickbottom_separator = '<span class="separator">|</span>';
    } /* ThemeRuta()

    /** construct an output page in HTML
     *
     * This constructs a full HTML-page, starting at the DTD
     * and ending with the html closing tag.
     *
     * The page is constructed using nested DIVs, the layout
     * is taken care of in separate style sheets. All knowledge
     * about the structure of the page is contained in this routine.
     *
     * As a rule the layout is based on three columns. The left hand
     * column contains free form HTML, the menu and more free form HTML.
     * The right hand column contains free form HTML, the contents of
     * 0, 1 or more html-pages and more free form HTML. The column in
     * the middle holds the actual content.
     *
     * The list of html-pages to show in the right hand column is
     * configurable per main menu item. A '0' (zero) means: no page,
     * a page number means that page, a section number means all
     * html-pages in that section and a dash ('-') means: suppress
     * the right hand side column completely. This also inserts an
     * additional stylesheet to style the 2-column layout.
     * Note that the stylesheets are added only once, first
     * 'style.css' and then 'style2.css'.
     *
     * This theme is also responsive. By default there are 2 tipping
     * points (at 650px and 980px) as can be seen in the default
     * 'style.css'. These tipping points also influence the background
     * image of the header and these are configurable. (The style
     * sheets should be edited manually).
     *
     * The header background changes every N minutes (N=configurable
     * between 0 and 60). Also, the choice of background image is
     * linked to the currently selected item in the main navigation.
     * There can be multiple sets of background images, each set can
     * be assigned to the header based on the width. The distribution
     * version uses three directories, for small (< 650), medium (650-980)
     * and large (>980).

     * The contents of the various DIVs is constructed in various
     * helper routines in order to make this routine easy to read
     * (by humans that is). The various helper routines all are called
     * with a string of space characters; this should improve the
     * the readability of the page that is generated eventually.
     *
     * Since this is a responsive theme, we have a choice regarding
     * these background banners; it is possible to configure a special
     * directory per size, e.g. one with small banners, one with medium
     * banners and one with large banners. This is configurable.
     * If one or more banners are selected, the appropriate CSS is
     * unconditionally added to the &lt;style&gt; header, including
     * the corresponding media selectors.
     *
     * Note that the routine $this->get_div_messages() does in fact
     * generate its own DIV tags. This is done in order to completely
     * get rid of the message DIV, we do not even want to see an empty
     * DIV if there are no messages. The same logic applies to the 
     * breadcrumb trail.
     *
     * This theme uses a tiny amount of Javascript to handle navigation
     * on small screens (<650): when space is tight a special 'menu'
     * button is inserted as the first item in the main navigation bar.
     * This link toggles between showing the #menu div and the top and
     * bottom HTML in the left margin. If JavaScript is NOT enabled,
     * the link jumps to the alternative menu #altmenu at the bottom of
     * the page. By default this altmenu is visible, but if Javascript
     * is enabled it is set to display none via code ({@see show_menu_js()}).
     *
     * @return string complete HTML-page, ready for output
     */     
    function get_html() {
        global $CFG;
        static $dejavu = 0;
        $navigation_index = $this->ruta_navigation_index();
        $sidebar_nodes = $this->ruta_sidebar_nodes_modules($navigation_index);
        // Initialise additional style (but only once)
        if ($dejavu++ == 0) {
            // if there is no sidebar at all, add a special 2-column stylesheet addendum
            if ($sidebar_nodes === FALSE) {
                if ((isset($this->config['style_usage_static'])) && 
                    ($this->config['style_usage_static']) &&
                    (isset($this->config['stylesheet2']))) {
                    $this->add_stylesheet($this->config['stylesheet2']);
                }
            }
            // Try to compute a background style for the header
            $this->ruta_header_background = $this->ruta_get_background($navigation_index);
        }

        // Finally construct actual page
        $s  = $this->dtd."\n".
              "<html lang=\"".$CFG->language_key."\">\n".
              "<head>\n".
                $this->get_html_head('  ').
              "</head>\n".
              "<body>\n".
              "  <div id=\"top\">\n".
                   $this->get_popups($this->messages_top,'    ').
              "  </div>\n".
              "  <div id=\"page\">\n".
              "    <div id=\"header\">\n".
              "      <div id=\"logo\">\n".
                       $this->get_logo('        ').
              "      </div>\n".
              "      <h1>".$this->title."</h1>\n".
              "      <h2>".$this->config['header_text']."</h2>\n".
              "    </div>\n".

              "    <div id=\"navigation\">\n".
                     $this->get_navigation('      ',$this->text_only).
              "    </div>\n".

              "    <div id=\"quicktop\">\n".
                     $this->get_quicktop('    ').
              "    </div>\n".
                   $this->get_div_breadcrumbs('    ').
                   $this->get_div_messages('    ').

              "    <div id=\"content\">\n".
              "      <div id=\"leftmargin\">\n".
              "        <div id=\"leftmargin_top\">\n".
                         $this->config['left_top_html']."\n".
              "        </div>\n".
              "        <div id=\"menu\">\n".
                         $this->get_menu('          ').
              "          <div id=\"menunav\">\n".
                           $this->get_menunavigation('            ',$this->text_only). // nested in #menu
              "          </div>\n".
              "        </div>\n".
              "        <div id=\"leftmargin_bottom\">\n".
                         $this->config['left_bottom_html']."\n".
              "        </div>\n".
              "      </div>\n".

              "      <div id=\"content_middle\">\n".
              "        <h2 id=\"content_title\">".htmlspecialchars($this->node_record['title'])."</h2>\n".
                       $this->get_content('        ').
              "      </div>\n";

        if ($sidebar_nodes !== FALSE) { $s .= 
              "      <div id=\"rightmargin\">\n".
              "        <div id=\"rightmargin_top\">\n".
                         $this->config['right_top_html']."\n".
              "        </div>\n".
              "        <div id=\"sidebar\">\n".
                         $this->ruta_get_sidebar($sidebar_nodes,'          ').
              "        </div>\n".
              "        <div id=\"rightmargin_bottom\">\n".
                         $this->config['right_bottom_html']."\n".
              "        </div>\n".
              "      </div>\n";
        }

        $s .= "    </div>\n".
              "    <div id=\"altmenu\">\n".
                     $this->get_altnavigation('      ',$this->text_only).
              "    </div>\n".

              "    <div id=\"footer\">\n".
              "      <div id=\"quickbottom\">\n".
                       $this->get_bottomline('        ').
              "      </div>\n".
              "    </div>\n".
              "    <div id=\"address\">\n".
                     $this->get_address('      ').
              "    </div>\n".
              "  </div>\n".
                 $this->show_menu_js('  ').
              "  <div id=\"bottom\">\n".
                   $this->get_popups($this->messages_bottom,'    ').
              "  </div>\n".
              "</body>\n".
              "</html>\n";
        return $s;
    } // get_html()


    /** construct the submenu starting at $menu_id OR the first breadcrumb in the top level menu
     *
     * this constructs an 'infinitely' nested set of submenus, starting at $menu_id
     * or at the first breadcrumb in the top level menu (if any).
     * If there are no suitable nodes, an empty string is returned.
     *
     * this is largely the same routine as parent::get_menu(). Difference is
     * that here we may add a menu title to the menu IF the first item in
     * the breadcrumb trail is a visible section
     *
     * @param string $m left margin for increased readability
     * @param int $menu_id indicates where to start the menu (NULL = first breadcrumb in top level menu)
     * @return string properly indented ready-to-use HTML
     * @uses show_tree_walk()
     */
    function get_menu($m='',$menu_id=NULL) {
        if (is_null($menu_id)) { // locate the toplevel section to open (if any)
            $node_id = $this->tree[0]['first_child_id'];
            for ( ; ($node_id != 0); $node_id = $this->tree[$node_id]['next_sibling_id']) {
                if ($this->tree[$node_id]['is_breadcrumb']) {
                    if (!($this->tree[$node_id]['is_page'])) {
                        $menu_id = $this->tree[$node_id]['first_child_id'];
                    }
                    break;
                }
            }
        }
        if (is_null($menu_id)) { // still nothing? we're done here
            return '';
        }
        $parent_id = $this->tree[$menu_id]['parent_id'];
        if ($parent_id == 0) {
            $menu = t('menu',$this->domain);
        } else {
            $menu = t('menu_menu',$this->domain,
                      array('{MENU}' => htmlspecialchars($this->tree[$parent_id]['record']['link_text'])));
        }
        return $m."<h3>".$menu."</h3>\n".
                   $this->show_tree_walk($m,$menu_id);
    } // get_menu()


    /** show footer text, maybe some quicklinks and 'powered by'
     *
     * @param string $m left margin for increased readability
     * @return string performance report
     */
    function get_bottomline($m='') {
        $separator = rtrim(' '.$this->quickbottom_separator); // readability
        $s = (empty($this->config['footer_text'])) ? '' : $m.$this->config['footer_text'].$separator."\n";
        $t  = $this->get_quickbottom($m);
        $s .= (empty($t)) ? '' : rtrim($t).$separator."\n";
        $s .= appropriate_legal_notices(TRUE,$m)."\n";
        return $s;
    } // get_bottomline()


    /** calculate the index of the current main navigation item
     *
     * this routine determines which main menu item is the current one,
     * starting at 1. if no menu item is current, the function returns 0.
     *
     * This index is used for two purposes:
     *  - calculating an ever changing background image for the header, and
     *  - determining which item of the comma-delimited pagenumbers to use for the sidebar.
     *
     * @return int index of current main menu item or 0 if none
     */
    function ruta_navigation_index() {
        $index = 0;
        $next_id = $this->tree[0]['first_child_id'];
        for ( ; ($next_id != 0); $next_id = $this->tree[$next_id]['next_sibling_id']) {
            if ($this->tree[$next_id]['is_visible']) {
                ++$index;
                if ($this->tree[$next_id]['is_breadcrumb']) {
                    return $index;
                    break;
                }
            }
        }
        return 0;
    } // ruta_navigation_index()


    /** compute the list of sidebar blocks for a main menu item
     *
     * this constructs an ordered array of node_id's corresponding
     * to the current main menu item. If the current main menu_item
     * indicates no sidebar then FALSE is returned.
     *
     * The basis for the node_ids to return is the contents of the
     * parameter 'sidebar_nodelist'. This is a comma-delimited string
     * containing node_ids, zeros or dashes.
     *
     * Example: suppose sidebar_nodelist contains "5,0,0,0,-".
     * If the first item of the main menu is current (ie. is part of
     * the breadcrumb trail), the node with node_id = 5 is examined.
     * If it is a page and the module is supported, this page's node_id
     * is returned. If, however, this is a section, all supported pages
     * within that section are returned.
     * For main menu items 2, 3 and 4 a '0' is specified. This means
     * that no pages will be displayed (an empty array is returned).
     * For main menu item 5 a dash ('-') is specified. This yields
     * a return value of FALSE which must be interpreted as 'suppress
     * the complete 3rd column of the page, including rightmargin_top
     * and rightmargin_bottom. This effectively reduces the 3-column
     * layout to a 2-column layout.
     *
     * Note that the node_ids in the sidebar_nodelist themselves must
     * not be under embargo but a hidden section (with visible pages)
     * is OK.
     *
     * Note that when there are more main menu items than entries in
     * the sidebar_nodelist, the value '0' is assumed.
     *
     * Finally, if no main menu item is current (eg. the visitor arrived
     * on a page via quicktop entries) then the value of '0' is assumed, too.
     *
     * @param int $index is the current main menu item or 0 for none
     * @return bool|array FALSE if no sidebar at all or an array node=>module pairs
     */
    function ruta_sidebar_nodes_modules($index) {
        $nodes = array();

        // 1 -- if no current menu item: no sidebar blocks
        if ($index <= 0) {
            return $nodes; 
        }
        $sidebar_nodelist = explode(',',$this->config['sidebar_nodelist']);

        // 2 -- if no entry for this current menu item: no sidebar blocks either
        if (!isset($sidebar_nodelist[$index-1])) {
            return $nodes;
        }

        // 3 -- if the entry starts with a dash: indicate no 3rd column at all
        if (substr(trim($sidebar_nodelist[$index-1]),0,1) == '-') {
            return FALSE;
        }

        // 4 -- if the entry is zero: once again: no sidebar blocks
        if (($node_id = intval($sidebar_nodelist[$index-1])) <= 0) {
            return $nodes;
        }

        // 5 -- if the node does not exist: no sidebar blocks
        if (!isset($this->tree[$node_id])) {
            return $nodes;
        }

        // 6 -- OK, finally go look for visible pages with supported modules (currently only 'htmlpage')
        $modules = get_module_records();
        if ($this->tree[$node_id]['is_page']) {
            // 6A -- single visible page?
            if ($this->tree[$node_id]['is_visible']) {
                $module_id = $this->tree[$node_id]['record']['module_id'];
                if ($modules[$module_id]['name'] == 'htmlpage') {
                    $nodes[$node_id] = $module_id;
                }
            }
        } else { // section
            // 6B -- multiple visible pages in a possibly hidden section
            // Note: if the section is under embargo, any pages within it are under embargo too
            // this is done via the page's 'is_visible' property.
            $next_id = $this->tree[$node_id]['first_child_id'];
            for ( ; ($next_id != 0); $next_id = $this->tree[$next_id]['next_sibling_id']) {
                if (($this->tree[$next_id]['is_page']) && ($this->tree[$next_id]['is_visible'])) {
                    $module_id = $this->tree[$next_id]['record']['module_id'];
                    if ($modules[$module_id]['name'] == 'htmlpage') {
                        $nodes[$next_id] = $module_id;
                    }
                }
            }
        }

        // 7 -- return array of module_ids keyed by node_id (but could be empty)
        return $nodes;
    } // ruta_sidebar_nodes_modules()


    /** retrieve data for sidebar 
     *
     * this retrieves the content of the various sidebar blocks (if any)
     * currently only the htmlpage module is supported.
     * 
     * @param array $sidebar_nodes contains node-module pairs to show
     * @param string $m add to human readability
     * @return string ready-to-use HTML-code
     */
    function ruta_get_sidebar($sidebar_nodes,$m='') {
        $s = '';
        $modules = get_module_records();
        if (!empty($sidebar_nodes)) {
            $index = 0;
            foreach($sidebar_nodes as $node_id => $module_id) {
                ++$index;
                switch ($modules[$module_id]['name']) {
                case 'htmlpage':
                    $s .= $this->ruta_get_sidebar_htmlpage($index,$node_id,$m);
                   break;
                }
            }
        }
        return $s;
    } // ruta_get_sidebar()


    /** retrieve page data for htmlpage module on a node
     *
     * this retrieves data from the page $node_id (requires knowledge of
     * internals of htmlpage module/table) in order to show that in a box
     * in the 3rd column.
     * The content is wrapped in a div which has a unique ID of the form
     * sidebar_blockN where N counts from 1 upward and a common class
     * sidebar_htmlpage. This allows for connecting style information to
     * a particular sidebar block in a flexible way.
     *
     * @param in $index block number (1-based)
     * @param int $node_id identifies the pagedata to retrieve 
     * @param string $m increased readbility
     * @return string ready-to-use HTML-code
     */
    function ruta_get_sidebar_htmlpage($index,$node_id,$m='') {
        $attributes = array('id' => 'sidebar_block'.strval($index),'class' => 'sidebar_htmlpage');
        $s = $m.html_tag('div',$attributes)."\n";
        // fetch actual content from database (requires knowledge of internals of htmlpage module/table)
        $table = 'htmlpages';
        $node_id = intval($node_id);
        $where = array('node_id' => $node_id);
        $fields = array('page_data');
        if (($record = db_select_single_record($table,$fields,$where)) === FALSE) {
            logger(sprintf('%s: no pagedata (node=%d): %s',__FUNCTION__,$node_id,db_errormessage()));
        } else {
            $s .= $record['page_data']."\n";
        }
        $s .= $m."</div>\n";
        return $s;
    } // ruta_get_sidebar_htmlpage()

    /** compute background image(s) based on current time and index
     *
     * this routine returns a snippet of ready-to-use code to set a
     * background image for #header. The images are different depending
     * on the time of day and the $index parameter. The latter is thought
     * to be the position of the current item in the main navigation
     * (starting at 1) or 0 in case no item in the main navigation 
     * is current.
     *
     * Configuration is done via two parameters: the paths in
     * $this->config['header_banners_directories'] (one per line) 
     * and the interval in $this->config['header_banners_interval'].
     *
     * The interval is expressed in minutes. It means that a particular page
     * has the same background image for the duration of the interval and
     * another one in the next interval. The banners directories are supposed
     * to contain banner files of the correct dimensions, e.g. 650x71 (small),
     * 980x170 (medium) or 1440x240 (large).
     *
     * If the specified directory starts with a '/' it is assumed to start in
     * the data root, e.g. something like /areas/exemplum/banners. This will
     * eventually lead to a URL of the form 
     * /file.php/areas/exemplum/banners/mentha-banner.jpg using was_file_url()
     *
     * If the specified directory does NOT start with a '/' it is assumed to
     * be a directory relative to the directory where index.php and friends
     * live, unless it starts with 'program/' in which case it is a static dir
     * somewhere in the program hierarchy. This is done via was_url().
     *
     * Note: if files are found in a directory, the names are sort()ed
     * so there is a chance that the three files are the same ones independent
     * of the physical order in the file system directory.
     *
     * @param int $index indicates current main navigation item or 0 for none
     * @param bool $fully_qualified TRUE forces URLs with scheme, authority etc.
     * @param string $m margin added for readability
     * @return string ready to use style including STYLE-tags or an empty string
     * @uses was_file_url()
     * @uses was_url()
     * @uses utf8_validate
     */
    function ruta_get_background($index,$fully_qualified=FALSE,$m='    ') {
        global $CFG;

        // 0 -- sanity check
        $directories = trim($this->config['header_banners_directories']);
        $interval = intval($this->config['header_banners_interval']);
        if ((empty($directories)) || ($interval <= 0)) {
            return '';
        }
        // 1 -- construct a helper array with banner directories keyed by tipping point
        $tips = explode(',',$this->config['tipping_points']); // e.g. "650,980"
        $dirs = explode("\n",$directories); // e.g. "/small\n/medium\n/large\n"
        $paths = array();
        $d = 0;
        for ($t=0; (($t < sizeof($tips)) && ($d < sizeof($dirs))); ++$t) {
            $point = intval($tips[$t]);
            if ($point <= 0) {
                $paths[0] = $dirs[$d++];
            } else {
                if ($t == 0) { // first tipping point; use 0 as implied tipping point
                    $paths[0] = $dirs[$d++];
                }
                if ($d < sizeof($dirs)) {
                    $paths[$point] = $dirs[$d++];
                }
            }
        } // now $paths[0]='/small', $paths[650]='/medium', $paths[980]='/large'

        // 2 -- step through paths[], look for image files and add corresponding style information
        $style = '';
        foreach($paths as $tipping => $path) {
            // 2A -- sanity check
            $path = trim($path);
            if ((!utf8_validate($path)) || (strpos('/'.$path.'/','/../') !== FALSE)) {
                logger(sprintf("%s.%s(): invalid path '%s'; bailing out",__CLASS__,__FUNCTION__,$path));
                return '';
            }
            // 2B -- compute absolute location, without trailing /
            if (substr($path,0,1) == '/') {
                $full_path = $CFG->datadir.$path;
            } elseif (substr($path,0,8) == 'program/') {
                $full_path = $CFG->progdir.substr($path,7);
            } else {
                $full_path = $CFG->dir.'/'.$path;
            }
            if (substr($full_path,-1) == '/') { $full_path = substr($full_path,0,-1); }

            // 2C -- scan this directory for graphics files (skip thumbs)
            if (($handle = @opendir($full_path)) === FALSE) {
                logger(sprintf("%s.%s(): cannot open dir '%s'",__CLASS__,__FUNCTION__,$path),WLOG_DEBUG);
                continue;
            }
            $prefix_len = strlen(THUMBNAIL_PREFIX);
            $extensions = explode(',',str_replace(array('.',' '),'',$CFG->filemanager_images));
            $files = array();
            while (($entryname = readdir($handle)) !== FALSE) {
                $full_entryname = $full_path.'/'.$entryname;
                if (($entryname == '.') || ($entryname == '..') || ($entryname == 'index.html') ||
                    (is_link($full_entryname)) || (!(is_file($full_entryname)))) {
                    continue;
                }
                // 2D -- we are now fairly sure $entryname is a genuine file. check extension (if any)
                if (strpos($entryname,'.') === FALSE) {
                    $ext = '';
                } else {
                    $components = explode('.',$entryname);
                    $ext = utf8_strtolower(array_pop($components));
                    unset($components);
                }
                if ((array_search($ext,$extensions) === FALSE) || // not an allowable image file
                    (substr($entryname,0,$prefix_len) == THUMBNAIL_PREFIX) || // thumbnail
                    (($image_info = @getimagesize($full_entryname)) === FALSE)) { // not image
                    continue;
                }
                $files[] = $entryname;
            }
            closedir($handle);

            // 3 -- choose a file based on index and time of day
            if (($n = sizeof($files)) <= 0) { // no file
                continue;
            }
            if ($n == 1) {
                $entryname = $files[0]; // Looks like Ford T: choice of 1
            } else {
                sort($files); // see note
                $entryname = $files[($index + time() / ( 60 * $interval)) % $n];
            }
            if (substr($path,0,1) == '/') {
                $url = was_file_url($path.'/'.$entryname,$fully_qualified);
            } else {
                $url = was_url($path.'/'.$entryname,$fully_qualified);
            }
            // add background image, maybe sandwiched in a media query
            $style .= sprintf("%s%s#header { background-image: url('%s'); }%s\n",
                         $m,
                         ($tipping>0) ? sprintf("@media only screen and (min-width: %spx) { ",$tipping):'',
                         $url,
                         ($tipping > 0) ? ' }' : '');
        }
        return (empty($style)) ? '' : "  <style>\n$style  </style>\n";
    } // ruta_get_background()


    /** insert special rotating background banner in bazaar style style in header
     *
     * the background style is precomputed onceand stored in $this->ruta_header_background.
     * This extends the original routine in such a way that the calculated background
     * is inserted _before_ the used-defined BSSS. This means that the user can overrule
     * the background image information if she wants to.
     *
     * @param string $m margin for increased readability
     * @return string empty string or ready to use HTML with header and Bazaar Style Style
     */
    function get_bazaar_style_style($m='') {
        return $this->ruta_header_background.
               parent::get_bazaar_style_style($m);
    } // get_bazaar_style_style()


    /** construct a top level menu (navigation bar) with an additional menu button
     *
     * start with a menu-button (that can be made invisible via CSS) and subsequently
     * walk through the top level of the menu tree and create a link for each node.
     * the first real link gets a different class so we can keep the first real link
     * visible while hiding the others via CSS. This is used to create a navbar with
     * two links 'MENU' and 'HOME' on the small (<650px) screen.
     *
     * Note that this MENU-button uses a tiny snippet of Javascript to toggle the menu.
     * 
     * @param string $m left margin for increased readability
     * @param bool $textonly forces a text-type link even when a navigation image is stipulated
     * @return string properly indented ready-to-use HTML
     */
    function get_navigation($m='',$textonly=FALSE) {
        $navbar = $m."<ul>\n";
        $attributes = array('onclick'=>'toggle_menu();', 'title'=>t('menu_button_title',$this->domain));
        $navbar .= $m.'  '.html_tag('li',array('class' => 'navmenu'),
                             html_a('#altmenu',NULL,$attributes,t('menu_button',$this->domain)))."\n";
        $items = 0;
        $next_id = $this->tree[0]['first_child_id'];
        for ( ; ($next_id != 0); $next_id = $this->tree[$next_id]['next_sibling_id']) {
            if ($this->tree[$next_id]['is_visible']) {
                $class = ($items == 0) ? 'navhome' : 'navsub'; // first 'real' LI treated different in CSS
                if ($this->tree[$next_id]['is_breadcrumb']) {
                    $class .= ' current';
                    $attributes = array('class' => 'current');
                } else {
                    $attributes = NULL;
                }
                $navbar .= $m.'  '.html_tag('li',array('class' => $class),
                           $this->node2anchor($this->tree[$next_id]['record'],$attributes,$textonly))."\n";
                ++$items;
            }
        }
        $navbar .= $m."</ul>\n";
        return $navbar;
    } // get_navigation()


    /** construct a top level menu (navigation bar)
     *
     * walk through the top level of the menu tree and create a link for each node.
     * This variant of the top level navigation is to be displayed under the submenu
     * in the left hand margin in case the screen is small (<650). In that case the
     * main navigation bar is limited to 'MENU' and 'HOME' and the rest of the main
     * navigation has to be available elsewhere. Different entries here are all members
     * of the class 'menunavsub'.
     * 
     * @param string $m left margin for increased readability
     * @param bool $textonly forces a text-type link even when a navigation image is stipulated
     * @return string properly indented ready-to-use HTML
     */
    function get_menunavigation($m='',$textonly=FALSE) {
        $navbar = $m."<ul>\n";
        $items = 0;
        $next_id = $this->tree[0]['first_child_id'];
        for ( ; ($next_id != 0); $next_id = $this->tree[$next_id]['next_sibling_id']) {
            if ($this->tree[$next_id]['is_visible']) {
                $class = 'menunavsub';
                if ($this->tree[$next_id]['is_breadcrumb']) {
                    $class .= ' current';
                    $attributes = array('class' => 'current');
                } else {
                    $attributes = NULL;
                }
                $navbar .= $m.'  '.html_tag('li',array('class' => $class),
                           $this->node2anchor($this->tree[$next_id]['record'],$attributes,$textonly))."\n";
                ++$items;
            }
        }
        $navbar .= $m."</ul>\n";
        return ($items > 0) ? $m.html_tag('h3',NULL,t('navigation',$this->domain))."\n".$navbar : "\n";
    } // get_menunavigation()

    /** create alternative navigation in case Javascript is not available
     *
     * This creates separated DIVs, one for each item in the main navigation bar, each containing 
     * the corresponding submenu. This whole alternative navigation DIV is set to display: none
     * using Javascript. If there is no Javascript, it remains visible. The submenus are displayed
     * for up to 2 levels deep, unless the current breadcrumb trail leads deeper into the tree.
     * This way most of the important nodes are reachable.
     *
     * @param string $m left margin for increased readability
     * @param bool $textonly forces a text-type link even when a navigation image is stipulated
     * @return string properly indented ready-to-use HTML
     */
    function get_altnavigation($m='',$textonly=FALSE) {
        $altnav = '';
        $next_id = $this->tree[0]['first_child_id'];
        for ( ; ($next_id != 0); $next_id = $this->tree[$next_id]['next_sibling_id']) {
            if ($this->tree[$next_id]['is_visible']) {
                $altnav .= $m."<div class=\"altmenu_column\">\n";
                $attributes = ($this->tree[$next_id]['is_breadcrumb']) ? array('class'=>'current') : NULL;
                $altnav .= $m.'  '.html_tag('h4',NULL,
                           $this->node2anchor($this->tree[$next_id]['record'],$attributes,$textonly))."\n";
                if (($subtree_id = $this->tree[$next_id]['first_child_id']) > 0) {
                    $altnav .= $this->alt_show_tree_walk($m.'  ',$subtree_id,2); // 2 levels deep at most
                }
                $altnav .= $m."</div>\n";
            }
        }
        if (!empty($altnav)) {
            $altnav = $m."<h3>".t('navigation',$this->domain)."</h3>\n".$altnav;
        }
        return $altnav;
    } // get_altnavigation()
        

    /** workhorse for constructing alternative recursive menu (walk the tree)
     *
     * construct nested (sub)menus for a depth of at most $max_level, unless
     * we hit the breadcrumb trail in which case we carry on nesting.
     * The (sub)menus are constructed in the form of nested UL's with LI's.
     *
     * The level of recursion of the list items (LI) is indicated via class='levelNNN'.
     * The type of item is indicated via class='page' or class='section'.
     * Also, the item has an addional class='current' when it is part of the breadcrumb trail.
     * Finally a current item also has the additional class 'activepage' or 'activesection' which
     * makes the CSS easier.
     *
     * The actual A-tag of the link only indicates being part of the breadcrumb trail via class='current'.
     *
     * It is up to the style sheet to visualise these items taking all variants into account.
     * Note that we only process visible pages and sections.
     *
     * Note: this routine looks very much like {@see show_tree_walk()}. The difference is the
     * limit $max_level.
     * 
     * @param string $m left margin for increased readability
     * @param int $subtree_id indicates where to start this (sub)menu
     * @param int $max_level maximum nesting displayed if not breadcrumbs
     * @return string properly indented ready-to-use HTML
     */
    function alt_show_tree_walk($m='',$subtree_id,$max_level) {
        static $level = 0;
        $class_level = 'level'.strval($level);
        $s = $m."<ul>\n";
        $node_id = $subtree_id;
        for ( ; ($node_id != 0); $node_id = $this->tree[$node_id]['next_sibling_id']) {
            if ($this->tree[$node_id]['is_visible']) {
                // 1 -- show this node
                $is_page        = $this->tree[$node_id]['is_page'];
                $is_breadcrumb  = $this->tree[$node_id]['is_breadcrumb'];
                $class = (($is_page) ? 'page ' : 'section ').$class_level;
                if ($is_breadcrumb) {
                    $class  .= (($is_page) ? ' activepage current' : ' activesection current');
                    $attributes = array('class' => 'current');
                } else {
                    $attributes = NULL;
                }
                $s .= $m.'  '.html_tag('li',array('class' => $class)).
                              $this->node2anchor($this->tree[$node_id]['record'],$attributes)."\n";

                // 2 -- maybe descend to subtree
                if (!$is_page) { 
                    if (($subsubtree_id = $this->tree[$node_id]['first_child_id']) > 0) {
                        ++$level;
                        if ($level > MAXIMUM_ITERATIONS) {
                            logger(__FILE__.'('.__LINE__.') too many levels in node '.$node_id);
                        } elseif (($is_breadcrumb) || ($level < $max_level)) {
                            $s .= $this->alt_show_tree_walk($m.'  ',$subsubtree_id,$max_level);
                        }
                        --$level;
                    }
                }
            }
        }
        $s .= $m."</ul>\n";
        return $s;
    } // alt_show_tree_walk()


    /** insert code for toggling the menu/free HTML in the left hand margin
     *
     * @return string ready-to-use Javascript-code sandwiched between script-tags
     */
    function show_menu_js($m='') {
        $js = $m."<script type=\"text/javascript\">\n".
              $m." function toggle_menu() {\n".
              $m."  t=document.getElementById('leftmargin_top');\n".
              $m."  m=document.getElementById('menu');\n".
              $m."  b=document.getElementById('leftmargin_bottom');\n".
              $m."  if (m.style.display=='block') {\n".
              $m."   t.style.display='block';\n".
              $m."   m.style.display='none';\n".
              $m."   b.style.display='block';\n".
              $m."  } else {\n".
              $m."   t.style.display='none';\n".
              $m."   m.style.display='block';\n".
              $m."   b.style.display='none';\n".
              $m."  }\n".
              $m."  return false;\n".
              $m." }\n".
              $m." document.getElementById('altmenu').style.display = 'none';\n".
              $m."</script>\n";
        return $js;
    } // show_menu_js()

} // ThemeRuta

?>